/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/**
 * Title:        Log utility for Java code
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Nira Shavitt
 * @version 1.0
 *
 * Usage: This class provides methods to be used for logging messages during
 * application execution.
 * The log levels is one of log4j levels: DEBUG, INFO, WARN, ERROR, FATAL, LOG.
 * Logging a message is done by invoking the proper lo4j method, for example:
 *   Log.logCategory.debug(message);
 *
 * Other methods to be used:
 * initCfg(fileName) - configure the logging (on/off and levels) from an
 *                  external file. see log.cfg in this directory.
 * initCfg(Properties) - configure the logging from Properties.
 * initCfg(logToConsole, logToFile, logFileName) - Configure the logging using the parameters:
 *        logToConsole - true, allows logging to System.out
 *        logToFile - true, allows logging to log file
 *        logFileName - name of the log file
 *
 * isLogging() - retrieves the information whether a logging is done or not.
 * printStackTrace(Exception e) - prints the stack trace to the previously
 * configured log channels.
 *
 *
 * A developer should import the following class, as follows, to be able to
 * compile code that invokes the Log methods:
 *
 * import com.radview.util.log.Log;
 *
 */

package com.radview.util.log;

import java.util.Properties;
import java.util.Date;
import java.io.FileOutputStream;
import java.io.PrintStream;

// log4j classes
import org.apache.log4j.Category;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.FileAppender;

// Radview classes
import com.radview.wlutil.TVEnv;


public class Log {

	 /*
   // logging levels, coordinated with log4j levels
   static public final int DEBUG = Priority.DEBUG_INT;
   static public final int INFO = Priority.INFO_INT;
   static public final int WARN = Priority.WARN_INT;
   static public final int ERROR = Priority.ERROR_INT;
   static public final int FATAL = Priority.FATAL_INT;
	 */

  // category to define whether the logging is enabled or disabled
  static public Category onOffCategory = null;

  // category to define the logging level
  static public Category logCategory = null;

  // boolean to define whether to actual log the message or not.
  static public boolean isLogging = false;

	static private boolean logToConsole = false;
	static private boolean logToFile = false;
	static private PrintStream logPrintStream = null;

	/* for webload5.0, comment this static block */
	static {
		/* use this code if we want to separate log files.
		   This code causes Log to be dependent in JRPConstants,
			 and we want to avoid that.

		// recorder is loading the class
		if (JRPConstants.RECORDER_LOG_FILE != null) {
			initCfg(true, // log to System.out
							true, // log to file
							JRPConstants.RECORDER_LOG_FILE); // file name
		}
		else { //if player is loading the class -> use TVEnv to init the file name
			if (JRPConstants.PLAYER_LOG_FILE == null) {
				JRPConstants.PLAYER_LOG_FILE = TVEnv.getTVEnvVar("PLAYER_LOG_FILE");
				initCfg(true, // log to System.out
								true, // log to file
								JRPConstants.PLAYER_LOG_FILE); // file name
			}
		}
		*/

		/*
		// init the log file name for the player
		if (LogFile.logFileName == null) { // was not init from the html (in the recording phase)
			LogFile.logFileName = TVEnv.getTVEnvVar("TRACE_FILE_PREFIX") + "_JRPPlayer.log";
		}

		initCfg(true, // log to System.out
						true, // log to file
						LogFile.logFileName); // file name
    */

		/* set the log configuration file */
		if (LogConfigFile.fileName == null) { // was not init from the html (in the recording phase)
		  try {
				// init the log configuration file name for the player
			  LogConfigFile.fileName = TVEnv.getTVEnvVar("TV_BIN") + java.io.File.separator + "javalogconfig.ini";
		  }
			catch (Exception e) { //ignore excpetions on TVEnv.
				System.out.println("Exception on TVEnv.getTVEnvVar(\"TV_BIN\")");
				System.out.println("Using configuration file: " + LogConfigFile.fileName);
				LogConfigFile.fileName = "javalogconfig.ini";
			}			
		}
		
		/* set the log directory  */
		if (RDFileAppender.getLogDir() == null) { // was not init from the html (in the recording phase)
		  try {
				// init the log configuration file name (for the player in JRP)
			  RDFileAppender.setLogDir(TVEnv.getTVEnvVar("LOG_DIR"));
		  }
			catch (Exception e) { //ignore excpetions on TVEnv.
				System.out.println("Exception on TVEnv.getTVEnvVar(\"LOG_DIR\")");
			  RDFileAppender.setLogDir(java.io.File.separator + "log" + java.io.File.separator);
				System.out.println("Using log dir: " + RDFileAppender.getLogDir());
			}			
		}
		
		/* load the log configuration file */
		initCfg(LogConfigFile.fileName);

		/* Each class Category is defined in the configuration file, for example:
	     log4j.category.com.radview.util.log.LogFile=DEBUG
			 Read the categories from the configuration file.                      */

		/* define whether we log or not ? */
		onOffCategory = Category.getInstance(Log.class);
    isLogging = onOffCategory.isDebugEnabled();

		/* define the log level (DEBUG,INFO,WARN,ERROR,FATAL) */
	  logCategory = Category.getInstance(LogConfigFile.class);

		if (isLogging()) {
			Date date = new Date(System.currentTimeMillis());
			logCategory.debug("\n\n\nLog (priority is "+ logCategory.getPriority()+") is started on "+date);
		}
		else {
		  System.out.println("Logging for Java is not enabled");
		}

	} // static

  /**
	 * Config the categories and appenders using an external configuration file.
	 * Invoked from the static block of this class.
	 */
  static public void initCfg (String configFileName) {

		try {
      PropertyConfigurator.configure(configFileName);
		}
		catch (Exception e) { // any runtime exception
			System.out.println("Failed to load log configuration file: "+configFileName);
			e.printStackTrace();
			// keep running
		}
  }

  /**
	 * Config the categories and appenders using parameters.
	 */
  static public void initCfg (boolean _logToConsole, boolean _logToFile, String logFileName) {
		try {
			PatternLayout patternLayout = new PatternLayout("[%t] %p %l - %m%n");

			if (_logToConsole)	{
				/* add ConsoleAppender to configuration */
				/* console Appender is set by default to System.out */
				ConsoleAppender consoleAppender = new ConsoleAppender(patternLayout);
				BasicConfigurator.configure(consoleAppender);
				logToConsole = true;
			}

			if (_logToFile) {
				/* add FileAppender to configuration */
				FileAppender fileAppender = new FileAppender(patternLayout,
				                                             logFileName,
																										 true); // append, not truncate
				BasicConfigurator.configure(fileAppender);

				/* construct printStream for logging exception in Log.printStackTrace() */
				FileOutputStream fileOutputStream = new FileOutputStream(logFileName);
				logPrintStream = new PrintStream(fileOutputStream, true);

				logToFile = true;
			}

			isLogging = onOffCategory.isDebugEnabled();
		}
		catch (Exception e) { // any runtime exception
			System.out.println("Exception on init log utility");
			e.getMessage();
			// keep running
		}
  }

  /**
	 * Config the categories using properties.
	 */
  static public void initCfg (Properties properties) {

      PropertyConfigurator.configure(properties);
      isLogging = onOffCategory.isDebugEnabled();
  }

	/* Commented, as when data like (line number, class name etc) is required,
	   this method outputs the same line and class, as taken from this method, and
		 does not output the line in the code that invokes this method.
  static public void logMsg(int logLevel, String message) {

     // log by the debug level
     if (logLevel == DEBUG)
       logCategory.debug(message);
     else
       if (logLevel == INFO)
         logCategory.info(message);
     else
       if (logLevel == WARN)
         logCategory.warn(message);
     else
       if (logLevel == ERROR)
         logCategory.error(message);
     else
       if (logLevel == FATAL)
         logCategory.fatal(message);
  }
	*/

	/**
	 * Returns the status of the logging, whether it is enabled or not.
	 * Apparently, invoking isLogging() is cheaper than using isLogging directly
	 */
  static public boolean isLogging() {
     return isLogging;
  }

	/**
	 * Prints the Stack Trace to log channels (System.out, log file) that
	 * were previously configured using initCfg(boolean, boolean, filename).
	 *
	 * IMPORTANT: Notice that this method requires initialization with
	 * initCfg(boolean, boolean, String), and not would not work when
	 * initCfg(filename) was invoked for initialization.
	 *
	 */
	static public void printStackTrace(Exception e) {
		if (logToConsole) {
			e.printStackTrace();  // flush the stack to System.err
		}

		if (logToFile) {
		  e.printStackTrace(logPrintStream);
		}
	}

   public static void main(String[] args) {

     // init the logging service using the configuration file.
     Log.initCfg(args[0]); // args[0] is the path to configuration file

     //Log.initCfg(true, true, args[1]); // args[0] is the path to log file

		 Log.logCategory.debug("Debug message");

		 Log.logCategory.info("Info message");

		 /*
		 try{
			throwException();
		 }
		 catch(Exception e) {
			 RVException rve = new RVRunTimeException("RVException test");
			 rve.setBaseException(e);

			 Log.logCategory.debug("Log test 1");
			 //Log.printStackTrace(e);
			 Log.logCategory.debug(rve.getFullStackTrace());
			 Log.logCategory.debug("Log test 2");
			 //Log.logCategory.debug(RVRunTimeException.stackTraceToString(e));
			 //Log.printStackTrace(e);
		 }
		 */

		 /*
     long startTime = System.currentTimeMillis();
     for (int i = 0; i < 100000000; i++) {
       if (Log.isLogging) {
         Log.logMsg(Log.DEBUG, "main: Entering application.");
       }
     }

     long endTime = System.currentTimeMillis();
     long time = endTime - startTime;
     if (Log.isLogging) {
       Log.logMsg(Log.DEBUG, "main: second log msg.");
     }
     System.out.println("cunsumed "+time+" milliseconds with logMsg()");
     logCategory.info("logging directly");
		 */
  } // main


	/* used for test only
	public static void throwException() throws Exception
	{
		Exception e = new Exception("test exception");

		throw e;
	}
	*/
}
