/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

 
/*
 *
 * Copyright (c) 1996 Brian Maso. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted. 
 * 
 * A Clock is a type of simple Synch. It is initiallized with a
 * half-period, in milliseconds. The Clock is initially unsignaled.
 * In a continunous loop, the Synch's state is changed between
 * signaled and unsignaled every half-period.
 *
 * The Clock works by creating a new Thread which simply waits
 * for the appropriate amount of time in a continuous loop.
 * When the Clock is destroyed, the Clock's thread reacts
 * appropriately by destroying itself.
 *
 */

package com.radview.util.synch;

/**
 * A Clock is a Synch which alternates its state between signaled and
 * unsignaled every <I>half_period</I> milliseconds. Clocks are always
 * created initially unsignaled. Note that the minimum effective period
 * of a Clock depends on the system your application is running. On more
 * powerful systems, the half-period can be as low as 50 milliseconds.
 * Less powerful machines can not handle those speeds.
 *
 * <P>A single thread
 * can potentially wait for and be released from a Clock several times
 * during a single cycle. The TickingClock Synch, which derives from the
 * Clock class, differs from the Clock by ensuring a single thread will
 * only be released once per period.
 *
 * @see TickingClock
 *
 * @version 1.0
 * @author Brian Maso
 */

public class Clock extends Synch implements Runnable {
	long _half_period;
	Thread _thread;
	static int __nNextThreadID = 0;
	
	/**
	 * You only need to specify the half-period of the Clock's cycle
	 * to the Clock class constructor. The <I>half_period</I> parameter
	 * specifies half the duration of a cycle of the Clock.
	 *
	 * @param	half_period
	 *					Every <I>half_period</I> millseconds the
	 *					Clock will switch states between signaled and
	 *					unsignaled.
	 */
	public Clock(long half_period) {
		super(unsignaled);
		_half_period = half_period;
		_thread = new Thread(this,
				"Clock-" + __nNextThreadID++ );
		_thread.setDaemon(true);
		_thread.start();
	}
	
	/* *********
	 * void gain(Thread)
	 *
	 * The Synch class implementation of this method
	 * automatically stores an OWNERSHIP flag for this
	 * Thread. Clocks, however, have no concept of ownership,
	 * and so should get rid of this flag.
	 *
	 ********* */
	
	void gain(Thread t) {
		super.gain(t);
		_hashOwnership.remove(t);
	}
	
	/* *********
	 * void run()
	 *
	 * The Thread created by this object runs within this method.
	 * The job of this method is to simply wait _half_period
	 * milliseconds, and switch Synch state if that time has
	 * elasped without the Clock having been destroyed. This
	 * Thread knows the Clock has been destroyed because the
	 * _thread value of this object will be null.
	 *
	 ********* */
	
	public void run() {
		// wait for _half_period milliseconds in a loop
		while(null != _thread) {
			try {
				Thread.currentThread().sleep(_half_period);
			} catch(Exception e) {}
			if(null == _thread)
				continue;
		
			// Switch Synch state before looping.
			setState(signaled==getState() ? unsignaled :
					signaled);
		}
		
		return;
	}
	
	/* *********
	 * void destroy()
	 *
	 * Overrider of the Synch.destroy() method. After calling
	 * Synch implementation, the _thread variable is set to null.
	 * When the running thread is awakened, the null value lets
	 * the Thread know this Timer has been destroyed.
	 *
	 ********* */
	
	/**
	 * Destroy the Clock when you are done using it. This is true of
	 * all Synch objects. Note that even after being destroyed, the
	 * Clock's background thread may linger for a little while. However,
	 * it will destroy itself automatically in time, and it will not effect
	 * the graceful shutdown of your application.
	 */
	public void destroy() {
		_thread = null;
		super.destroy();
	}
}
