/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

 
/*
 *
 * Copyright (c) 1996 Brian Maso. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted. 
 * 
 * A Signal is a simple Synch. It provides methods to make the Signal
 * signaled or unsignaled externally. Each of these methods properly
 * calls Synch.setState() to modify the object's state.
 *
 */

package com.radview.util.synch;

/**
 * The Signal class is the simplest type of Synch. A Signal's
 * state is set externally using the <I>signal()</I> and <I>unsignal()</I>
 * methods. Here is an example:
 *
 * <PRE>
 *	public class Example extends Frame implements Runnable {
 *		Thread _background = new Thread(this);
 *		Signal _signal = new Signal();
 *
 *		public static void main(String[] astrArgs) {
 *			resize(50, 50);
 *			setLayout(new BorderLayout());
 *			add("Center", new Button("Start"));
 *			show();
 *			return;
 *		}
 *
 *		public void run() {
 *			while(Scheduler.ABANDONED != Scheduler.waitFor(_signal)) {
 *				System.out.println("iteration...");
 *			}
 *			return;
 *		}
 *
 *		public boolean handleEvent(Event evt) {
 *			if(Event.WINDOW_DESTOY == evt.id) {
 *				_signal.destroy();
 *				dispose();
 *                      System.exit(0);
 *				return true;
 *			}
 *
 *			if(Event.ACTION_EVENT == evt.id) {
 *				_signal.signal();
 *				return true;
 *			}
 *
 *			return super.handleEvent(evt);
 *		}
 *	}
 *
 * </PRE>
 *
 * A Signal is not an "owned" Synch.
 *
 * @version 1.0
 * @author Brian Maso
 */

public class Signal extends Synch {

	/**
	 * Signal is created in the unsignaled state
	 * by default.
	 */
	public Signal() {
		super();
	}
	
	/**
	 * Pass in the initial state of the Signal. Possible initial
	 * states are <I>Synch.signaled</I> and <I>Synch.unsignaled</I>.
	 *
	 * @param	state		<UL>The initial state has one of two possible
	 *				values:
	 *				<LI> <I>Synch.signaled</I> - initial state is
	 *				unsignaled. This is the default.
	 *				<LI> <I>Synch.signaled</I> - initial state is
	 *				signaled.
	 */
	public Signal(int state) {
		super(state);
	}
	
	/**
	 * Set the Signal's state to signaled.
	 */
	public void signal() {
		signal(true);
	}
	
	/**
	 * Set the Signal's state to unsignaled.
	 */
	public void unsignal() {
		signal(false);
	}
	
	/**
	 * Set the Signal's state to either signaled or
	 * unsignaled.
	 *
	 * @param	fSignal		If true, then the Signal's
	 *					state is set to signaled. If
	 *					false, then it is set to unsignaled.
	 */
	public void signal(boolean fSignal) {
                setState(fSignal ? signaled : unsignaled);
	}
	
	/* *********
	 * void gain(Thread)
	 *
	 * The Synch class implementation of this method
	 * automatically stores an OWNERSHIP flag for this
	 * Thread. Signals, however, have no concept of ownership,
	 * and so should get rid of this flag.
	 *
	 ********* */
	
	void gain(Thread t) {
		super.gain(t);
		_hashOwnership.remove(t);
	}
}
