/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************************
*
*   Author  : Yossi Nakash
*   Date    : 25.01.20001
*   File    : CallbackData.h
*   Purpose :
*   Classes : CLocation, CHttpHeaders, CUrlData, CUrlDataEx, CallbackData.
*
*******************************************************************************/
#ifndef _CALLBACK_DATA_H
#define _CALLBACK_DATA_H

#include "wlhtmlvirtualqueue.h"
#include "wltransactiondata.h"
#include <tvslist.h>

class DllExport CLocation;
class DllExport CHttpHeaders;
class DllExport CUrlData;
class DllExport CUrlDataEx;
class DllExport CallbackData;

#define MIME_TYPE_DEFAULT "text/plain"

class CWLDocument ;

/********************************************************************
  Name:    CLocation.
  Purpose: This class contains a pointer to the location, offset to
           that location in the Frame, and it's length.
           The location is actually a pointer to a RVCString data in
           the object of that location. (Image, Link, etc.)
********************************************************************/

class CLocation
{
public:
  CLocation(const char* pLocation, int nOffset, int nLength);
  CLocation(const char* pLocation, int nOffset, int nLength, bool bIsImage);
  virtual ~CLocation();

  CLocation( const CLocation& src );
  CLocation& operator=( const CLocation& src );

public:
  const char* m_pcLocation ;  // href, url, we do not allocate memory for it.
  int   m_nOffset ;           // offset of the url in the file(buffer received)
  int   m_nLength ;           // length of the url sent
  bool  m_IsImage ;

};


/********************************************************************
  Name:    CHttpHeaders.
  Purpose: This class contains the Request and Response headers.
********************************************************************/
class CHttpHeaders
{
public:
  CHttpHeaders();
  virtual ~CHttpHeaders();

  CHttpHeaders( const CHttpHeaders& src );
  CHttpHeaders& operator=( const CHttpHeaders& src );

  const char* GetRequestHeader();
  const char* GetResponseHeader();
  void  SetRequestHeader(const char* sHeader);
  void  SetResponseHeader(const char* sHeader);

protected:
  RVCString m_pcRequestHttpHeader ;
  RVCString m_pcResponseHttpHeader ;

};


/********************************************************************
  Name:    CUrlData.
  Purpose: This class contains the basic url data.
********************************************************************/
class CUrlData
{
public:
  CUrlData();
  virtual ~CUrlData();

  void SetUrl(const char* pcUrl);
  void SetOriginalUrl(const char* pcOrigUrl);
  void SetReferer(const char* pcReferer);
  void SetFinalUrl(const char* pcFinalUrl);
  void SetTarget(const char* pcTarget);
  void SetDom(CWLDocument* pDom);
  void SetGetFrames(bool getFrames);
  void Reset();

public:
  const char* m_pcUrl ;     //url in the first http Get
  const char* m_pcOrigUrl ; //Original url (relative locations etc.)
  const char* m_pcReferer ;
  const char* m_pcFinalUrl ;// the final get in case of redirections.updated in EndPage and EndObject
  char* m_pcTarget ;
  CWLDocument* m_pDom ;     //a pointer to the dom, Sent in EndPage only.
  bool        m_GetFrames;

private:
  // avoid copy ctor & asignment operator
  CUrlData( const CUrlData& src );
  CUrlData& operator=( const CUrlData& src );

};


/********************************************************************
  Name:    CUrlDataEx.
  Purpose: This class inherits from class CUrlData, and contains in
           addition the httpStatus, and the urlType.
********************************************************************/
class CUrlDataEx : public CUrlData
{
public:
  CUrlDataEx();
  virtual ~CUrlDataEx();

  void SetHttpStatus(int stat);
  void SetUrlType(HtmlQueueItemType eUrlType);
  void Reset();

public:
  int         m_nHttpStatus ;
  HtmlQueueItemType m_eUrlType ;//enumeration of url types. declaration is in: wlhtmlvirtualqueue.h

private:
  // avoid copy ctor & asignment operator
  CUrlDataEx( const CUrlDataEx& src );
  CUrlDataEx& operator=( const CUrlDataEx& src );

};


/********************************************************************
  Name:    CallbackData.
  Purpose: This class inherits from class CUrlDataEx, and contains in
           addition the httpStatus, and the urlType.
********************************************************************/
class CallbackData : public CUrlDataEx
{
public:
  CallbackData();
  virtual ~CallbackData();

  void AddRequestHeader(const char* Header);
  void AddResponseHeader(const char* Header);
  void AddLocation(const char* pLocation, int nOffset, int nLength, bool bIsImage = false);
  void Reset();
  void SetMimeType(const char* pcMimeType);
  void SetObjContent(void* pContent);
  void SetObjContentLength(int nLength);
  void SetFileSize(int nFileSize);
  void SetCheckSum(CHECKSUM_VAR checksum);

public:

  const char* m_pcMimeType ;
  void*       m_pContent ;  //!!!!This is a pointer to the buffer. If you want to use it, you have to copy it !!!
  int         m_nContentLength ;
  int         m_nFileSize ;
  CHECKSUM_VAR m_CheckSum ;

  RVTValSlist<CLocation> LocationList;  //contains all locations
  RVTValSlist<CHttpHeaders> HeadersList;  //contains Http Request & Response headers

private:
  // avoid copy ctor & asignment operator
  CallbackData( const CallbackData& src );
  CallbackData& operator=( const CallbackData& src );

};



#endif // _CALLBACK_DATA_H
