/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : HttpResponse.cpp
 *
 *	Author : Zion Menahem
 *
 *	Description : Based on the HTTPUAR module, this file implements
 *	              the HTTP protocol for JavaScript module.
 *
 *	History : 
 *
 *	Modifications : 
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/

/** include files **/
#include <math.h>
#include "rvjsapi.h"
#include <RVCString.h>
#include <wlerrorlib.h>
#include "wlqueue.h"
#include "httpdefines.h"
#include "httprequest.h"
#include "httpresponse.h"
#include "httpcookies.h"
#include "jshttpnames.h"
#include "commandmanager.h"
#include "workspace.h"
#include "tracedll.h"

#include "httpheaderhandler.h"
#include "httpdefines.h"
#include "wlhttpcache.h"

/** local definitions **/
#define HEADER_TRANSFER_CODING   "transfer-coding"
#define HEADER_TRANSFER_ENCODING "transfer-encoding"
#define TOKEN_CHUNKED            "Chunked"

#define STATUS_LINE_KEY "Status line"

#define SP          " "
#define LF          "\012"
#define CR          "\015"
#define CRLF_SIZE   2
#define LF_SIZE			1
#define CRLFx2      "\015\012\015\012"
#define CRLFx2_SIZE 4


#define TEXT_HTML  "text/html"

#define READ_SIZE_ERROR     10
#define CHUNK_SIZE_ERROR    53
#define GOT_LAST_CHUNK      12
#define GOT_CHUNK_SIZE      13
#define CHUNK_IN_PROCESS    20
#define CHUNK_ERROR         54
#define EXTRACT_NEXT_CHUNK  22
#define EXTRACT_FOOTER_DONE 30
#define EXTRACT_HEADER_OK   31
#define ENTITY_HEADER_ERROR 55
#define NEED_MORE_DATA      40
#define READ_NEXT_BUFFER    41

#define min(a,b) (((a) < (b)) ? (a) : (b))
#define max(a,b) (((a) > (b)) ? (a) : (b))


bool IsXml (const RVCString & contentType, const RVCString& uri)
{
  bool IsXmlPage = false ;

  //Search for Xml extention (ignore query string if exsists)
  size_t QueryStringPos = uri.index("?", 0) ;
  if (QueryStringPos == RV_NPOS) QueryStringPos = uri.length();
  IsXmlPage = uri.index(".xml", QueryStringPos-4, RVCString::ignoreCase) == (QueryStringPos-4) ? true : false;					
  if (IsXmlPage) return IsXmlPage ;

  //Search for Xml content types
  IsXmlPage = contentType.index("text/xml", 0, RVCString::ignoreCase) == 0 ? true : false;					
  if (IsXmlPage) return IsXmlPage ;

	IsXmlPage = contentType.index("application/xml", 0, RVCString::ignoreCase) == 0 ? true : false;					

  return IsXmlPage ;
}

//
// Functions for CWLHttpResponse class.
CWLHttpResponse::CWLHttpResponse() 
  : output(NULL), status(STATUS_LINE), m_ContentLength(INIT_CONTENT_LENGTH), server_status(0),
    line ((RVSize_T)4096)
{
	content_type = "";
	m_TransferType = "";
	m_Version = VERSION10;
	m_Status = INIT_STATUS;
	m_Reason = "";
	m_KeepAlive = false;
	m_ContResponse = false;
  m_bCallback = false ;
  m_bCheckFileContent = false ;
  m_eZipContentCode = ZipCode_NO;
	m_bIsFirstContentChunk = true;
  m_cpHttpRequest = NULL;
  m_lengthForParseOnly = 0;

  // Chaim
  m_UrlFirstTime = false;
  m_UrlChecked = false;
  m_QueueRestored = false;
}

CWLHttpResponse::~CWLHttpResponse()
{
  m_cpHttpRequest = NULL;
  clearAndDestroy();
  if (output)
    delete output;
}

void CWLHttpResponse::Init(CWLHttpRequest* p_cHttpRequest)
{
	m_Version = VERSION10;
	m_Status = INIT_STATUS;
	m_Reason = "";

  
  m_cpHttpRequest = p_cHttpRequest;
  
	status = STATUS_LINE;
	m_ContentLength = INIT_CONTENT_LENGTH;
	server_status = 0;
	m_ContResponse = false;
  m_lengthForParseOnly = 0;

	if (!m_TransferType.isNull())
		m_TransferType.remove(0);
	if (!content_type.isNull())
		content_type.remove(0);
	if (!line.isNull())
		line.remove(0);
	if (entries() != 0)
		clearAndDestroy();

  // Chaim Apr. 2001
  m_bHeaderContentLength = false;
  m_bHeaderLocation = false;
  m_bHeaderAuthenticate = false;
  m_bHeaderProxyAuthenticate = false;
  m_bHeaderConnection = false;

  // Chaim
  m_UrlFirstTime = false;
  m_UrlChecked = false;
  m_QueueRestored = false;
	m_ChunkedData.Init();
  m_eZipContentCode = ZipCode_NO ;
}

int CWLHttpResponse::setOutput(const char* file)
{
  if (output) {
    delete output;
    output = NULL;
  }
  if (file)
    output = new ofstream(file, ios::out | ios::binary);
#ifdef __TV_WINDOWS__
	if (output  && output->is_open())
#else
	if (output)
#endif
		return RET_OK;
	else
		return RET_ERROR;
}

void CWLHttpResponse::resetOutput()
{
  if (output) {
    delete output;
    output = NULL;
  }
}


const char* CWLHttpResponse::GetHeader(const char* key)
{
	if (key == NULL  ||  key[0] == '\0')
		return NULL;

	myCollectableStr  index(key);
	index.toLower();
	RVCollectableString* value;
	value = (RVCollectableString *)findValue(&index);
	if (!value)
		return (NULL);
	else
		return (value->data());
}


void CWLHttpResponse::printHeaderValues()
{
  RVHashDictionaryIterator iter(*this);
  while (iter()) {
    cout << "key = " << ((myCollectableStr *)iter.key())->data() << "    "
         << "value = " << ((RVCollectableString *)iter.value())->data()
         << endl;
  }
}


int CWLHttpResponse::findStatusNumber(RVCString& data)
{
  int i = data.index(HTTPPROTOCOL, 0, RVCString::ignoreCase);
  if (i == RV_NPOS  || i != 0)
    return (RV_NPOS);
  else  {
    int j = data.index(" ", i);
    if (j == RV_NPOS)
      return (RV_NPOS);
    else 
      return (j+1);
  }

}


size_t CWLHttpResponse::findDelimiter(const char* data, int* size)
{
	const char* pos = strstr(data, CRLF);
	if (pos) {
		*size = CRLF_SIZE;
		return (int)((unsigned long)pos - (unsigned long)data);
	}
	else
		return RV_NPOS;
}


void CWLHttpResponse::addHeader(RVCString& aKey, RVCString& aValue)
{
	myCollectableStr* key = new myCollectableStr(aKey);
	RVCollectableString* value = new RVCollectableString(aValue);
	if (!insertKeyAndValue(key, value)) 
  {
		delete key;
		delete value;
	}
  else 
  {
		m_sHeaders += aKey ;
		m_sHeaders += ": ";
		m_sHeaders += aValue ;
		m_sHeaders += CRandLF;
    WlTraceMsg( "Header: %s:%s", aKey.data(), aValue.data());
  }
}

 
void CWLHttpResponse::AppendHeader(RVCString& aKey, RVCString& aValue)
{
	myCollectableStr* key = new myCollectableStr(aKey);

	RVCollectableString* value;
	value = (RVCollectableString *)findValue(key);

  if (!value) //value does not exist, we add it to header.
  {
    addHeader(aKey, aValue);
  }
	else
  {
    ((RVCString*)value)->append('\n');
    aValue = ((RVCString*)value)->append(aValue);
    WlTraceMsg( "Header: %s:%s", aKey.data(), ((RVCString*)value)->data());
  }

  delete key;
}

int CWLHttpResponse::getVersion(RVCString& buff)
{
	RVCString ver(HTTPPROTOCOL);
	ver += "1.1";
	if (buff.index(ver, 0, RVCString::ignoreCase) == 0)
		return VERSION11;

	return VERSION10;
}

bool CWLHttpResponse::IsStatusLine(RVCString piToken)
{
  piToken.strip(RVCString::leading);
  if ((piToken.index(HTTPPROTOCOL,0,RVCString::ignoreCase) == 0)) 
    return true ;
  else
    return false ;
}

bool CWLHttpResponse::extractVersion()
{
  /*
	line = line.strip(RVCString::leading);
	if (line.index(HTTP_1_1, 0, RVCString::ignoreCase) == 0) {
		m_Version = VERSION11;
		line.remove(0, strlen(HTTP_1_1));	// remove version string
	}
	else if (line.index(HTTP_1_0, 0, RVCString::ignoreCase) == 0) {
		m_Version = VERSION10;
		line.remove(0, strlen(HTTP_1_0));	// remove version string
	}
	else
		return false;

	return true;
  */
  
  /*
  // Chaim
  size_t index = line.index(HTTP_1_1, 0, RVCString::ignoreCase);
	if (index != RV_NPOS) {
		m_Version = VERSION11;
		line.remove(0, index + strlen(HTTP_1_1));	// remove version string
	}
  else {
    index = line.index(HTTP_1_0, 0, RVCString::ignoreCase);
    if (index != RV_NPOS) {
		  m_Version = VERSION10;
		  line.remove(0, index+ strlen(HTTP_1_0));	// remove version string
    }
  	else
	  	return false;
	}
  */

  int minVer = -1;
  char proto[64];
  memset(proto,0,64);
	if(sscanf(line.data(), "%[^/]/1.%d",proto, &minVer) != 2)
		return false;
  
  int protoLen = strlen(proto);
  switch(minVer) 
  {
  case 0: 
           m_Version = VERSION10;
           break;
  case 1:
           m_Version = VERSION11;
           break;
  default:
    return false;
  }

  line.remove(0, protoLen + 4);

	return true;
  
}

bool CWLHttpResponse::extractStatus()
{
	line = line.strip(RVCString::leading);
	if (sscanf(line.data(), "%u", &m_Status) != 1)
		return false;

	line.remove(0, 3);	// remove status number
	return true;
}

// Chaim - strip current line to next CrLf.
void CWLHttpResponse::stripToCrLf()
{
	size_t crlfPos = line.index(LF);
	size_t CrLf_Size = LF_SIZE;
  line.remove(0, crlfPos + CrLf_Size);	
}

void CWLHttpResponse::extractReason()
{
	line = line.strip(RVCString::leading);
	size_t crlfPos = line.index(LF);
	size_t CrLf_Size = LF_SIZE;

	// First we check if we found a LF in line
	if (crlfPos != RV_NPOS) {
		// If we found then we check if it is not on the start
		// of the line
		if (crlfPos > 0) {
			// If it is not at start of line then we check if before the
			// LF we have CR
			if (line.index(CR)==crlfPos-1) {
				CrLf_Size = CRLF_SIZE;
				crlfPos--;
			}
		}
		// if we are not on the start of the line then we
		// extract the reason
		if (crlfPos > 0) {
			m_Reason = line(0, crlfPos);
		}
		// In any case we extract the reason+the LF or CRLF
		line.remove(0, crlfPos + CrLf_Size);	// remove reason & <CRLF>
	}
  
}
/*
void CWLHttpResponse::extractReason()
{
	line = line.strip(RVCString::leading);
	size_t crlfPos = line.index(CRLF);
	if (crlfPos != RV_NPOS && crlfPos != 0) {
		m_Reason = line(0, crlfPos);
		line.remove(0, crlfPos + CRLF_SIZE);	// remove reason & <CRLF>
	}
}
*/


int CWLHttpResponse::ParseResponse(CWLTransactionsObject* p_pTransObj,
                                   CWLDocument* frame, 
																	 const char* buffer, 
																	 int len, 
																	 int* done,
																	 CWLCookies* Cookietree, 
																	 HttpFuncs method,
																	 int version,
																	 CWLUserParseFlags::ParseOptions doParse, 
																	 bool saveSource,
        													 bool saveHeaders,
                                   bool ParseOnce,
																	 RVCString& host, 
																	 RVCString& uri, 
																	 HtmlQueueItemType urlType,
																	 int threadNum, 
																	 CWLCommandManager* cm, 
                                   bool ExplicitUrl, 
																	 CWLHttpCacheEntry* p_pCacheEntry,
																	 int& errCode,
                                   RVCString& sBuffer)
{
  int         i = 0;
	int					internalErrCode;
	size_t      crlfPos,CrLf_Size, colPos, crlfx2Pos;
  const char* buff = NULL;
	int         bufflen = 0;
  bool        XmlContent = false;

  // Chaim
  RVCollectableString key(host + uri);

  // Chaim - for patch to support ParseOnce for embedded frames/xml
  bool  ParseGeneral = false;

//  RVCollectableString key(uri);

	errCode = 0;
	internalErrCode = 0;

  // Chaim - skip appending of buffer is done with header and parseOnce.
  m_lengthForParseOnly += len;

  if (status == MESSAGE_BODY &&
      !IsCallbackEnabled() &&
      ParseOnce &&
      m_QueueRestored && // means we've read up to the point where we have restored
                         // the saved transactions from the cache
      !saveSource
      ){
    *done = m_lengthForParseOnly;
    return RET_OK;
  }
  
	line.append(buffer, len);

  switch (status) {
    case STATUS_LINE:
			m_bIsFirstContentChunk = true;
			// extract status line, loop on discarded responses (like '100 continue')

			do {
				if (m_ContResponse) {
					crlfx2Pos = line.index(CRLFx2);
					if (crlfx2Pos == RV_NPOS)
						return RET_OK;

					line.remove(0, crlfx2Pos + CRLFx2_SIZE);
					m_ContResponse = false;
				}
      // if we haven't seen at least LF at this state
				// we return and wait for more data to come
				crlfPos = line.index(LF);
				if (crlfPos == RV_NPOS) {
	          return RET_OK;
				}
				else {
					if (crlfPos > 0) {
						if (line.index(CR)==crlfPos-1) {
							crlfPos--;
						}
					}
				}

/*				Old version
				crlfPos = line.index(CRLF);
				if (crlfPos == RV_NPOS)
					return RET_OK;
*/

				status = MESSAGE_HEADER;

				RVCString sline = line(0, crlfPos);
        RVCString sStatusLine (sline);
				int in = findStatusNumber(sline);

				// extract version
				if (!extractVersion())	// check if status line is legal (starts with 'HTTP/...')
					break;	// try the header block

				// the final m_Verion value is set as following:
				// request | response | final
				// --------+----------+------
				// 1.0     | 1.0      | 1.0
				// 1.0     | 1.1      | 1.0
				// 1.0     | other    | 1.0
				// 1.1     | 1.0      | 1.0
				// 1.1     | 1.1      | 1.1
				// 1.1     | other    | 1.0
				if (m_Version != VERSION11 || version != VERSION11)
					m_Version = VERSION10;

				if (m_Version == VERSION10)
					m_KeepAlive = false;
				else
					m_KeepAlive = true;

				// extract the status number
				extractStatus();
				if (m_Status == INIT_STATUS)	// no status exist
					break;	// try the header block

				if (urlType == FRAME && frame)
					frame->insertStatus(m_Status, in, sline);

				// when a 'continue' response received, we should ignore it since at this
				// point we already sent the whole request !
				if (m_Status == 100) {
					m_ContResponse = true;
					status = STATUS_LINE;
          continue;
        }
        else {
          if (urlType == FRAME && frame) 
					  frame->insertHeader(STATUS_LINE_KEY, sStatusLine.data());
        }

        // Chaim - extract reason only if callback enabled.
				if (IsCallbackEnabled())
				extractReason();
        else
          stripToCrLf();

WlTraceMsg( "Status line: HTTP/%u.%u %u %s", m_Version/10, m_Version%10, m_Status, m_Reason.data());
			} while (m_ContResponse);
      
    case MESSAGE_HEADER:

			// when new content (status != 304) is received for a cached object, it's content should be erased
			if ( p_pCacheEntry && m_bIsFirstContentChunk )
			{
				m_bIsFirstContentChunk = false;
      // (not all) relevant headers are retransmitted, so reset them
				p_pCacheEntry->Reset();

				// clear the old content if modified (not not-modified)
				if ( m_Status != 304 )
				{
					//cm->GetHttpCache().ContentAdded( -p_pCacheEntry->GetContentLength() );
					p_pCacheEntry->ClearContent();
				}
			}

      // Chaim - set flag when header is first read
      // Chaim - patch for embedded frames and xml islands:
      // If the urlType is GENERAL, which is how the ParseOnce mechanism saves the
      // transaction objects it saves to avoid parsing, then we check if a backup exists
      // for it. Unlike FRAME or XML, if there's no backup for it, then we don't try to
      // create one.
      if (!m_UrlChecked && ParseOnce && !IsCallbackEnabled()){
        if (urlType == FRAME || urlType == XML){
        if (!cm){
          WlTraceMsg("Cm is null");
          return RET_ERROR;
        }
        m_UrlFirstTime = !cm->BackupExists (&key);
        m_UrlChecked = true;
        m_QueueRestored = false;
      }
        else if (urlType == GENERAL && cm->BackupExists (&key)){
          WlTraceMsg("Found backup for embedded frame/xml");
          m_UrlFirstTime = false;
          m_UrlChecked = true;
          m_QueueRestored = false;
          ParseGeneral = true;
        }
      else {
          m_UrlChecked = true;
          m_UrlFirstTime = false;
          ParseGeneral = false;
          cm->SetBackup (false, &key);
        }
      }
      else {
        m_UrlChecked = true;
        m_UrlFirstTime = false;
        cm->SetBackup (false, &key);
        ParseGeneral = false;
      }

      // Chaim Apr. 16 2001 - Headers are stored only when SaveSource flag is set

      // storing the headers in RVOrdered object.
      while (status == MESSAGE_HEADER) {
				crlfPos   = line.index(LF);
				CrLf_Size = LF_SIZE;
  			if (crlfPos == RV_NPOS) {
	          return RET_OK;
				}
				else {
					if (crlfPos > 0) {
						if (line.index(CR)==crlfPos-1) {
							CrLf_Size = CRLF_SIZE;
							crlfPos--;
						}
					}
				}
				
        if (crlfPos == 0) {
          //This means that we reached two consecutive CrLf which is the 
          //formal sign to the end of the http header part.
          status = MESSAGE_BODY;
          line.remove(0, CrLf_Size);
          break; // from the while, but not from the case statement.
        }

/*			Old version
				crlfPos = line.index(CRLF);
				if (crlfPos == RV_NPOS)
          return RET_OK;

				if (crlfPos == 0) {
          status = MESSAGE_BODY;
          line.remove(0, CRLF_SIZE);
          break; // from the while, but not from the case statement.
        }
*/

				// extract the header(s)
        colPos = line.first(':');
        if (colPos == RV_NPOS || colPos > crlfPos) {
          //ignore repeating status line
          if ((colPos > crlfPos) && IsStatusLine(line(0,crlfPos))) {
            line.remove(0, crlfPos+CrLf_Size);
            continue;
          }
          else {
            status = MESSAGE_BODY;
          break;
          }
        }

        RVCString name = line(0, colPos);
        name = name.strip(RVCString::both, ' ');
				// check for at least one character after the colon
				RVCString value("");
				if (crlfPos - colPos > 1) {
					value = line(colPos + 1, crlfPos - (colPos + 1));	// skip ':'
					value = value.strip(RVCString::leading, ' ');
				}
				name.toLower();

				// check for special headers
        // Chaim
        CHLHeaderHandler::HttpHeaders HeaderId = CHLHeaderHandler::GetHeaderId(name.data());
        switch( HeaderId )
//        switch( CHLHeaderHandler::GetHeaderId(name.data()) )
        {
        case CHLHeaderHandler::EN_HEADER_COOKIE_RESPONSE : 
          {
          if (Cookietree->parseCookieIntoTree(value, 
                                              host, 
                                              uri, 
                                              cm->GetWorkspace()->m_CookieExpiration, 
                                              internalErrCode) == RET_ERROR) {
						errCode = internalErrCode;
					}

          // Chaim Apr. 16 2001 - Save headers only if required
					if (saveHeaders) 
            addHeader(name, value);
          break;
				}

        case CHLHeaderHandler::EN_HEADER_CONTENT_LENGTH :
          {
					// keep content-length header value for POST/GET only
					if (method != HTTP_FUNCS_HEAD ) {
	          m_ContentLength = atoi(value.data());
					}
          // Chaim Apr. 16 2001 - Save headers only if required and set flags for internal use
          m_bHeaderContentLength = true;
					if (saveHeaders) 
					  addHeader(name, value);
          break;
				}

        case CHLHeaderHandler::EN_HEADER_CONTENT_TYPE :
          {
          content_type = value;
          // Chaim Apr. 16 2001 - Save headers only if required
					if (saveHeaders) 
            addHeader(name, value);
			//Galtech
#ifndef _ANSI_ONLY
				if (urlType == FRAME && frame)
				    ParseContentType(frame, value, cm);
#endif
          break;
				}

        case CHLHeaderHandler::EN_HEADER_CONTENT_ENCODING :
          {
            if (value.compareTo("gzip",RVCString::ignoreCase ) == 0)
              m_eZipContentCode = ZipCode_GZIP;
            else if (value.compareTo("deflate",RVCString::ignoreCase ) == 0)
              m_eZipContentCode = ZipCode_DEFLATE;
            else // for now
              m_eZipContentCode = ZipCode_NO ;
            
            if (saveHeaders)
              addHeader(name, value);
            break;
          }

				case CHLHeaderHandler::EN_HEADER_DATE:
					{
						// add value to cache
						if ( p_pCacheEntry )
							p_pCacheEntry->SetDate( value.data() );
						if ( saveHeaders )
			        addHeader( name, value );
				    break;
					}

				case CHLHeaderHandler::EN_HEADER_ETAG:
					{
						// add value to cache
						if ( p_pCacheEntry )
							p_pCacheEntry->SetETag( value.data() );
						if ( saveHeaders )
			        addHeader( name, value );
				    break;
					}

				case CHLHeaderHandler::EN_HEADER_EXPIRES:
					{
						// add value to cache
						if ( p_pCacheEntry )
							p_pCacheEntry->SetExpires( value.data() );
						if ( saveHeaders )
			        addHeader( name, value );
				    break;
					}

				case CHLHeaderHandler::EN_HEADER_LAST_MODIFIED:
					{
						// add value to cache
						if ( p_pCacheEntry )
							p_pCacheEntry->SetLastModified( value.data() );
						if ( saveHeaders )
			        addHeader( name, value );
				    break;
					}

        case CHLHeaderHandler::EN_HEADER_TRANSFER_CODING :
          {
					m_TransferType = value;
          // Chaim Apr. 16 2001 - Save headers only if required
					if (saveHeaders) 
            addHeader(name, value);
          break;
				}

        case CHLHeaderHandler::EN_HEADER_TRANSFER_ENCODING :
          {
					m_TransferType = value;
          // Chaim Apr. 16 2001 - Save headers only if required
					if (saveHeaders) 
            addHeader(name, value);
          break;
				}

        // If it is an authentication header then we check that it one that we know
        // how to handle its authorization (Currently only Basic and NT)
        // We need to add here trace message if we find some header which we are 
        // not capable of handling
        case CHLHeaderHandler::EN_HEADER_AUTHENTICATE : 
          {
          if ((value.index(TOKEN_NTLM, 0, RVCString::ignoreCase) == 0) || 
              (value.index(TOKEN_BASIC, 0, RVCString::ignoreCase) == 0)) {

            // Chaim Apr. 16 2001 - Save headers only if required
//            AppendHeader(name, value);
	  				if (saveHeaders) 
              AppendHeader(name, value);
            m_bHeaderAuthenticate = true;
            m_HeaderAuthenticateValue = value;
          }
          else {
            WlTraceMsg( "Header: %s:%s Was not added - Unknown authentication mechanism", name.data(), value.data());
          }
          break;
        }

        // Chaim Apr. 2001 - Support for saveHeaders
        case CHLHeaderHandler::EN_HEADER_PROXY_AUTHENTICATE : 
          {
   				if (saveHeaders) 
            AppendHeader(name, value);
          m_bHeaderProxyAuthenticate = true;
          m_HeaderProxyAuthenticateValue = value;
          break;
        }

        // Chaim Apr. 2001 - Support for saveHeaders
        case CHLHeaderHandler::EN_HEADER_LOCATION : 
          {
   				if (saveHeaders)
            addHeader(name, value);
          m_bHeaderLocation = true;
          m_HeaderLocationValue = value;
          break;
        }

        case CHLHeaderHandler::EN_HEADER_CONNECTION : 
          {
   				if (saveHeaders)
            addHeader(name, value);
          m_bHeaderConnection = true;
          m_HeaderConnectionValue = value;
          break;
        }

        default:{
          // Chaim Apr. 16 2001 - Save headers only if required
					if (saveHeaders)
					addHeader(name, value);
				}
        }

				// update the keep-alive status
        // Chaim
				if (HeaderId == CHLHeaderHandler::EN_HEADER_CONNECTION) {
//				if (CHLHeaderHandler::GetHeaderId(name) == CHLHeaderHandler::EN_HEADER_CONNECTION) {
					if (m_Version == VERSION10 && CHLHeaderHandler::GetHeaderId(value.data()) == CHLHeaderHandler::EN_TOKEN_KEEP_ALIVE)
						m_KeepAlive = true;
					else if (m_Version == VERSION11 && CHLHeaderHandler::GetHeaderId(value.data()) == CHLHeaderHandler::EN_TOKEN_CLOSE)
						m_KeepAlive = false;
				}

        // Chaim Apr. 2001 - saveHeaders support - talk to Yossi about this one.
				if (urlType == FRAME && frame && saveHeaders)
					frame->insertHeader(name.data(), value.data());
        line.remove(0, crlfPos + CrLf_Size);
/* Old version
        line.remove(0, crlfPos + CRLF_SIZE);
*/
      }
      
    case MESSAGE_BODY:

      // Chaim
      m_UrlChecked = false;

			// when 'Content-length' header is missing it counts as close-connection
      // Chaim Apr. 16 2001 - Get header info from flags and not header store
			if (m_Version == VERSION10 && !m_bHeaderContentLength)
//			if (m_Version == VERSION10 && !GetHeader(HEADER_CONTENT_LENGTH))
				m_KeepAlive = false;

			// when request method was HEAD, receive should end here
			if (method == HTTP_FUNCS_HEAD)
				return RET_HEAD_DONE;

      if (!line.isNull()) {
        buff = (char*)line.data();
				bufflen = line.length();
      }

//if (urlType == FRAME)      
//WlTraceMsg( "before chunking data size: %d, buff:\n%s", line.length(), line.data());
			if (m_Version == VERSION11) {
				// extract chunk transfer coding

				// @@Chaim (WagerWorks) - the WagerWorks server sometimes sends the chunk size before the headers
				// My fix is to treat chunked messages w/o a size as unchunkled.
				WlTraceMsg("***** buf: %s", buff);
				int res;
				if (!m_bHeaderLocation && CHLHeaderHandler::GetHeaderId(m_TransferType.data())==CHLHeaderHandler::EN_TOKEN_CHUNKED &&
					((res = m_ChunkedData.extractMessage(buff, bufflen)) != READ_SIZE_ERROR)) {
//WlTraceMsg( "current data size: %d, buff: %s\n", line.length(), line.data());
					do {

						if (res == CHUNK_ERROR || res == ENTITY_HEADER_ERROR) {
							errCode = res;
							return RET_ERROR;
						}

						if (!line.isNull())
							line.remove(0);
						line.append(m_ChunkedData.getParseData(), m_ChunkedData.getParseSize());
						buff = (char*)line.data();
						bufflen = line.length();

						if (res == EXTRACT_FOOTER_DONE) {
							m_ContentLength = m_ChunkedData.getLength();
							m_ChunkedData.Init();
						}

						if (res == EXTRACT_HEADER_OK) {
              WlTraceMsg( "Header: %s:%s", m_ChunkedData.lastHeaderName(), m_ChunkedData.lastHeaderValue());
							// Chaim Apr. 16 2001 - Save headers only if required
              if (saveHeaders) 
              {
							addHeader(m_ChunkedData.lastHeaderName(), m_ChunkedData.lastHeaderValue());
							frame->insertHeader(m_ChunkedData.lastHeaderName(), m_ChunkedData.lastHeaderValue());
						}
            }
					} while (res == EXTRACT_HEADER_OK);
				}
      // Chaim Apr. 16 2001 - Get header info from flags and not header store
				else if (!m_bHeaderContentLength) {
//				else if (!GetHeader(HEADER_CONTENT_LENGTH)) {
					m_KeepAlive = false;
				}
			}

			// in case of redirection, content does not interest us (in parsing & outputting to file).
      // Chaim Apr. 16 2001 - Get header info from flags and not header store
      if (output && urlType == FRAME && !m_bHeaderLocation)
        output->write(buff, bufflen);

      // Chaim Apr. 16 2001 - Get header info from flags and not header store
      if (IsCallbackEnabled() && !m_bHeaderLocation){
        if (GetEZipCode() == ZipCode_NO){
          // Chaim - Zipped content is handled in httprequest. 
          m_Output.Write(buff, bufflen);
        }
     }
 
      if (IsCheckFileContent() && p_pTransObj)
      {
        p_pTransObj->AddChecksum(buff, bufflen);
        p_pTransObj->AddFileSize(bufflen);
      }

      // when parsing is needed, parsing is done according to the content-type (if exist).
			// only if the content-type does not exist, parsing is done according to the url type.
			// (url type does not overrides content-type.
			// we don't want to parse a page which is a redirection
//if (urlType == FRAME)      
//WlTraceMsg( "parser would get data size: %d, buff:\n%s", bufflen, buff);

			bool parsed = false;

      // Chaim - cached images issue in IDE
      // In callback mode, we save the content for images and stylehseets in the cache so that
      // we won't send an empty content to the IDE and it will display the images (due to fixes for earlier
      // bugs, IDE doesn't even try to look up images with empty content in its cache).
      // As part of this change, I also make sure to save the contet type in the cache (there was an interface for it
      // but it wasn't saved).
//      if ((urlType == IMAGE || urlType == STYLE) && IsCallbackEnabled() && p_pCacheEntry){
      if ((content_type.contains("image",  RVCString::ignoreCase) || urlType == STYLE) && IsCallbackEnabled() && p_pCacheEntry){
        if (bufflen > 0) {
          p_pCacheEntry->AppendContent( buff , bufflen );
        }
        p_pCacheEntry->SetContentType(GetContentType());
      	p_pCacheEntry->SetEZipCode( GetEZipCode() ); // currently the only other content-encoding supported
      }

			if (bufflen > 0 && frame) 
      {
				//xml is always "parsed" parse == get the src of the xml island

        // Chaim Apr. 16 2001 - Enter parse only if url type is actually
        // a frame or an XML island.
        if ((doParse) && (urlType == XML || urlType == FRAME ||
              (urlType == GENERAL && ParseGeneral))) {

					if ( p_pCacheEntry )
					{
						p_pCacheEntry->AppendContent( buff, bufflen );
                                                p_pCacheEntry->SetContentType(GetContentType());
						p_pCacheEntry->SetEZipCode( GetEZipCode() ); // currently the only other content-encoding supported
					}

//				if (doParse || urlType == XML) 
          ContentTypes enContentType = CheckContentTypes(cm, uri);              

          if ((urlType == FRAME && (enContentType != NOT_DEFINED || content_type.isNull())) || 
              (urlType == XML) ||
              (urlType == GENERAL && ParseGeneral)) 

              // Chaim Apr. 16 2001 - Get header info from flags and not header store
						if (!m_bHeaderLocation)
//						if (!GetHeader(HEADER_LOCATION)) 
            {
              // redirected

              //Patch for Xml direct parsing -- Added by Aviad
              if (enContentType == XML_TYPES && (urlType == FRAME)) 
                  frame->StartXmlDirectParsing();
    
              // Chaim - support for parse once
              if (ParseOnce && !IsCallbackEnabled())
							{
                if (!m_UrlFirstTime){
//                  cm->SetBackup (false, &key);
//                  frame->parse(buff, bufflen);  //temp
                  if (!m_QueueRestored){
                    cm->UseBackup (&key);
                    cm->SetBackup (false, &key);
                    m_QueueRestored = true;
                  }
                }
                else {
                  WlTraceMsg("Parsing document for 1st time");
                  cm->SetBackup (true, &key);
                  //if (GetEZipCode()==ZipCode_GZIP)
                  ZipTypeCode ZTC = GetEZipCode();
                  if ((ZTC==ZipCode_GZIP) || (ZTC==ZipCode_DEFLATE) || (cm->GetWorkspace()->m_bParseFullBuffer))
                  {
                    sBuffer.append(buff, bufflen);
                  }
                  else
                  {
              frame->parse(buff, bufflen);
                }
              }
              }
              else 
              {
                //if (GetEZipCode()==ZipCode_GZIP)
                ZipTypeCode ZTC = GetEZipCode();
                if ((ZTC==ZipCode_GZIP) || (ZTC==ZipCode_DEFLATE)|| (cm->GetWorkspace()->m_bParseFullBuffer))
                {
                  sBuffer.append(buff, bufflen);
                }
              	else 
              	{
                  // specifies authentication iteration.
                  if(m_Status == 401){
                    if(m_cpHttpRequest->GetAuthState())
              		    frame->parse(buff, bufflen);
                  }else{
                    frame->parse(buff, bufflen);
                  }
					      }
              }
							parsed = true;
						}
				  }
			  }

			// if saveSource is set, send the source to the parser to be saved
			// for now, only applicable for explicit requests (then urlType == FRAME)
			if (!parsed && saveSource && frame && urlType == FRAME)
				frame->SaveSource(buff, bufflen);

      if (!line.isNull())
        line.remove(0);

			*done = bufflen;
			return RET_OK;
      //return m_TotalDone;
  }
  return RET_ERROR;
}



ZipTypeCode  CWLHttpResponse::GetEZipCode()
{
  return m_eZipContentCode;
}

void CWLHttpResponse::SetEZipCode( ZipTypeCode p_eZipCode )
{
  m_eZipContentCode = p_eZipCode;
}




// ============================================================================

CWLChunkedData::CWLChunkedData()
{
	Init();
}


void CWLChunkedData::Init() {
	m_DoneSize = m_ChunkSize = m_TotalSize = 0;
	m_CurrState = CHUNK_SIZE;

	if (!m_FooterData.isNull())
		m_FooterData.remove(0);			
	if (!m_CurrData.isNull())
		m_CurrData.remove(0);			
	if (!m_ParseData.isNull())
		m_ParseData.remove(0);			
}

int CWLChunkedData::extractSize()
{
	// read data size
	int crlfPos = m_CurrData.index(CRLF);
	if (crlfPos != RV_NPOS) {
		if (sscanf(m_CurrData.data(), "%x", &m_ChunkSize) != 1)
			return READ_SIZE_ERROR;

		m_CurrData.remove(0, crlfPos + CRLF_SIZE);	// remove the whole chunk-size line

		if (m_ChunkSize == 0)
			return GOT_LAST_CHUNK;

		m_TotalSize += m_ChunkSize;
		return GOT_CHUNK_SIZE;
	}
	return NEED_MORE_DATA;
}


int CWLChunkedData::extractData()
{
	unsigned long sizeLeft = m_ChunkSize - m_DoneSize;
	if (sizeLeft > 0) {
		m_LastSize = min(sizeLeft, m_CurrData.length());
		m_ParseData.append(m_CurrData.data(), m_LastSize);
		m_CurrData.remove(0, m_LastSize);
		m_DoneSize += m_LastSize;
	}

	if (m_ChunkSize > m_DoneSize)
		return READ_NEXT_BUFFER;

	m_DoneSize = 0;
	m_ChunkSize = 0;
	return EXTRACT_NEXT_CHUNK;
}


int CWLChunkedData::extractHeader()
{
	// check for a complete header line
	size_t crlfPos = m_FooterData.index(CRLF);
	if (crlfPos == RV_NPOS)
		return NEED_MORE_DATA;

	if (crlfPos == 0)
		return EXTRACT_FOOTER_DONE;

	size_t name_end = m_FooterData.index(":");
	if (name_end == RV_NPOS)
		return ENTITY_HEADER_ERROR;

	m_LastHeaderName = m_FooterData(0, name_end);
	m_LastHeaderName = m_LastHeaderName.strip(RVCString::both, ' ');

	size_t value_start = name_end + 2;	// skip ':' and single ' '
	m_LastHeaderValue = m_FooterData(value_start, crlfPos - value_start);
//WlTraceMsg( "header: %s: %s", m_LastHeaderName, m_LastHeaderValue);

	m_FooterData.remove(0, crlfPos + CRLF_SIZE);

	return EXTRACT_HEADER_OK;
}

int CWLChunkedData::extractFooter()
{
	int res;

	m_FooterData += m_CurrData;
	if (!m_CurrData.isNull())
		m_CurrData.remove(0);
//WlTraceMsg( "footer: %s\n", m_FooterData.data());
	res = extractHeader();

	return res;
}


// go through all parts of the chunked body and collect the data to be parsed
int CWLChunkedData::extractMessage(const char* buff, int len)
{
	int res;

	m_CurrData.append(buff, len);
	if (!m_ParseData.isNull())
		m_ParseData.remove(0);

	while (true) {
		switch (m_CurrState) {
			case CHUNK_SIZE:
				res = extractSize();
				if (res == GOT_LAST_CHUNK) {	// when chunk-size == 0
					m_CurrState = FOOTER;
					break;
				}

				if (res == READ_SIZE_ERROR)	// something is wrong (not an HEX number ?)
					return res;

				if (res == NEED_MORE_DATA)	// chunk-size line lacks the <CRLF>
					return res;

				// res == GOT_CHUNK_SIZE
				m_CurrState = CHUNK_DATA;

			case CHUNK_DATA:
				res = extractData();
				if (res == READ_NEXT_BUFFER)	// data buffer ended before chunk's end
					return res;

				if (res == EXTRACT_NEXT_CHUNK)	// chunk-size bytes read
				{
					m_CurrState = CHUNK_DATA_END;
				}

				break;

			case CHUNK_DATA_END:
				if (m_CurrData.index(CRLF) == 0) {	// right termination of a chunk
					m_CurrData.remove(0, CRLF_SIZE);
					m_CurrState = CHUNK_SIZE;
					break;
				}

				// @@Chaim - fix for PetStore bug. I don't know why we SOMETIMES get this state but I think it's better
				// to simple be less strict.
				if (m_CurrData.length() > CRLF_SIZE)	// chunk should end but it ends with other than <CRLF> string
//					return CHUNK_ERROR;
					WlTraceMsg("Missing CRLF in chunked data?");

				return NEED_MORE_DATA;	// <CRLF> not found
				break;

			case FOOTER:
				res = extractFooter();

				if (res == EXTRACT_FOOTER_DONE)	// last of the headers read
					m_CurrState = CHUNK_SIZE;

				return res;
				break;
		}
	}
}

CWLHttpResponse::ContentTypes CWLHttpResponse::CheckContentTypes(CWLCommandManager* cm, const RVCString& uri)
{
  // all the content types that are considered as html.
  RVCString sContentType(cm->GetWorkspace()->m_HtmlContentTypes);
  RVCString ResponseContentType(cm->GetWorkspace()->m_ResponseContentType);
	RVCString sReceivedContenType(content_type);
  bool htmlContent = false, IsXmlPage = false ;
  int key ;

	if (ResponseContentType == RESPONSE_CONTENT_TYPE_HTML) {
		return HTML_TYPES ;
	}

	if (ResponseContentType == RESPONSE_CONTENT_TYPE_XML) {
		return XML_TYPES ;
	}

  //clean up "parameter" from content-type.
  if ( (key = content_type.first(';')) != RV_NPOS )
    sReceivedContenType.remove(key+1);

  if (!sContentType.isNull())
  {
    htmlContent = sContentType.index(sReceivedContenType, 0, RVCString::ignoreCase) == RV_NPOS ? false : true;
  }

  if (htmlContent)
    return HTML_TYPES ;

  //Check for XML
  size_t QueryStringPos = uri.index("?", 0) ;
  if (QueryStringPos == RV_NPOS)
    QueryStringPos = uri.length();
  if ( uri.index(".xml", QueryStringPos-4, RVCString::ignoreCase) == (QueryStringPos-4) )
  {
    return XML_TYPES ;
  }

  sContentType = cm->GetWorkspace()->m_XmlContentTypes ;
  if (!sContentType.isNull())
  {
    IsXmlPage = sContentType.index(sReceivedContenType, 0, RVCString::ignoreCase) == RV_NPOS ? false : true;
  }

  if (IsXmlPage)
    return XML_TYPES ;

  return NOT_DEFINED ;
}


RVCString& CWLHttpResponse::GetResponseHeadersString()
{
  return m_sHeaders;
}

void CWLHttpResponse::ResetResponseHeadersString()
{
  m_sHeaders.remove(0);
}

RVCString CWLHttpResponse::GetAndResetResponseHeaderString()
{
  RVCString headers(m_sHeaders) ;
  char cStatusLine[512] = "";
  sprintf(cStatusLine,"%s: HTTP/%u.%u %u %s%s", STATUS_LINE_KEY, m_Version/10, m_Version%10, m_Status, m_Reason.data(), CRandLF);
  ResetResponseHeadersString() ;
  headers.insert(0, cStatusLine);
  return headers ;
}

//Galtech
#ifndef _ANSI_ONLY
void CWLHttpResponse::ParseContentType(CWLDocument* pFrame, const RVCString& sValue, CWLCommandManager* cm)
{
	content_type = sValue;
	long lCharset;
	bool bEnforceGlobalCharEncoding = cm->GetWorkspace()->m_bEnforceGlobalCharEncoding;

	if (bEnforceGlobalCharEncoding)
	{
		// @@Chaim - CharEncoding change - always use loadengine default
//		lCharset = cm->GetWorkspace()->m_CharSet;
		lCharset = GetMonitorCharSet();
	}
	else
	{
		RVCString tmp = sValue;
		tmp.toUpper();
		char *ptr;
		const char *str = (const char*)tmp;	
		if (ptr = strstr(str, "CHARSET"))
		{
			ptr += strlen("charset");
			while (isspace(*ptr))
				ptr++;

			if (*ptr == '=')
			{
				ptr++;
				while (isspace(*ptr))
					ptr++;

				char pcCharset[32] = "";
				char *sptr = pcCharset;
				while (*ptr && !isspace(*ptr))
					*sptr++ = *ptr++;
				*sptr = '\0';
	 			lCharset = UL_CodePageNameToID(pcCharset);
			}
		}
		else
		{
			// default value
			// @@Chaim - CharEncoding change - always use loadengine default
//			lCharset = cm->GetWorkspace()->m_CharSet;
			lCharset = GetMonitorCharSet();
		}
	}

	//set current char set
	// @@Chaim - CharEncoding change: at this point, lcharset has been either found in page or taken from loadengine global
	pFrame->SetCharset((lCharset == CP_DEFAULT) ? GetMonitorCharSet() : lCharset, bEnforceGlobalCharEncoding);
}
#endif
