/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// SlaveCountManager.cpp: implementation of the CLoadAgendaStatisticsManager class.
//
//////////////////////////////////////////////////////////////////////
#include <sortvec.h>

#include "associativearray.h"
#include "tracedll.h"
#include "loadagendamanager.h"
#include "loadgenerator.h"
#include "loadagendastatisticsmanager.h"

class CStatisticsListItem :  public RVCollectable
{
  friend class CLoadAgendaStatisticsManager;
private:
  double              m_Time;
  long                m_CurrentCount;
  long                m_ExpectedCount;
  CAssociativeArray	 *m_Message;

public:
  CStatisticsListItem(double time);
  CStatisticsListItem(double time, long expectedCount);
  ~CStatisticsListItem();
  
  virtual bool isEqual(const RVCollectable* o) const;
  virtual int compareTo(const  RVCollectable* o) const;
  virtual char *nameOf() const;

  bool NewStatistics( CAssociativeArray& StatisticsMessage );
  double GetTime() const { return m_Time; }
  const char* GetStatisticsString() 
  {
		m_Message->AddCouple("$ENTRIES", m_CurrentCount);
    return m_Message->ToString(); 
  }
};

CStatisticsListItem::CStatisticsListItem(double time)
{
  m_Time = time;
  m_Message = NULL;
}

CStatisticsListItem::CStatisticsListItem(double time, long expectedCount)
{
  m_Time = time;
  m_ExpectedCount = expectedCount;
  m_CurrentCount = 0;
  m_Message = new CAssociativeArray;
}

CStatisticsListItem::~CStatisticsListItem()
{
  if (m_Message)
    delete m_Message;
}

bool CStatisticsListItem::NewStatistics( CAssociativeArray& StatisticsMessage )
{
  //
  // Update the Count.
  m_CurrentCount++;
  // Add a new statistics string.
  m_Message->AddArrayCouple("$STATSTRING", StatisticsMessage.GetValueOfParameter("$STATISTICSMANAGERDATA"), m_CurrentCount);
  //
  // Return true if all expected message arrived.
  if (m_CurrentCount>=m_ExpectedCount)
    return true;
  else
    return false;
}

bool CStatisticsListItem::isEqual(const RVCollectable* o) const
{
  return( ((CStatisticsListItem*)o)->m_Time == m_Time );
}

int CStatisticsListItem::compareTo(const  RVCollectable* o) const
{
  if( ((CStatisticsListItem *)o)->m_Time == m_Time )
    return 0;
  else if( ((CStatisticsListItem *)o)->m_Time < m_Time )
    return 1;		
  else
    return -1;
}

char *CStatisticsListItem::nameOf() const
{
  return ("SlaveCountListItem");
}


class CStatisticsListByTime :  public RVSortedVector
{
private:
  double              m_LastTimeSent;

public:
  CStatisticsListByTime() : m_LastTimeSent(0) {};
  ~CStatisticsListByTime() {};

  void SetLastTimeSent(double time) {m_LastTimeSent = time;};
  double LastTimeSent() {return m_LastTimeSent;};
};


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CLoadAgendaStatisticsManager::CLoadAgendaStatisticsManager()
{
  m_pMyLoadAgendaManager = NULL;
}

CLoadAgendaStatisticsManager::~CLoadAgendaStatisticsManager()
{
  m_StatisticsScriptsDict.clearAndDestroy();
}

void CLoadAgendaStatisticsManager::SetMyCLoadAgendaManager(CLoadAgendaManager* pLAM)
{
  m_pMyLoadAgendaManager = pLAM;
}

void CLoadAgendaStatisticsManager::NewStatistics( CAssociativeArray& StatisticsMessage )
{
	//  Lock since this method is called in each LGProxy
	RVMutexLock::LockGuard lock(m_ListLock);

  // get the time of the statistics transaction
  double time = StatisticsMessage.GetDoubleValueOfParameter("$TIME");

  // get the agenda name of the statistics message since it might be one of a mix
  RVCollectableString agendaName = StatisticsMessage.GetValueOfParameter("$AGENDANAME");

  WlTraceMsg("CLoadAgendaStatisticsManager::NewStatistics(time=%f, agendaName=%s)", time, agendaName);

	// get the statistics list (for each time) for that agenda
  CStatisticsListByTime* pStatisticsListByTime = (CStatisticsListByTime*)m_StatisticsScriptsDict.findValue((RVCollectable*)&agendaName);
  if (!pStatisticsListByTime) {
		// if does not exists create a new list
    pStatisticsListByTime = new CStatisticsListByTime();
    m_StatisticsScriptsDict.insertKeyAndValue(new RVCollectableString(agendaName.data()), pStatisticsListByTime);
  }
  CStatisticsListByTime& StatisticsListByTime = *pStatisticsListByTime;
  CStatisticsListItem *item = NULL;

	// This is done for 2 reasons:
	//	1) The Statistics Generator always creates transaction for the first time resolution
	//			even if the slave started later.
	//	2) ????
  if (time <= StatisticsListByTime.LastTimeSent()) {
    WlTraceMsg(" \n \n*** CLoadAgendaStatisticsManager::NewStatistics - Error with time: (%f,%f\n \n ",time, m_LastTimeSent);
    return;
  }

	// looking for the entry of the current time (the entry include all statistics data from
	// all LGs from that time)
	//  ??? why from the end to the begining ???
  CStatisticsListItem temp(time);
  item = (CStatisticsListItem*)StatisticsListByTime.find(&temp);
  /*
  for( int i = StatisticsListByTime.entries()-1; i >= 0; i-- )
  {
    CStatisticsListItem *tempitem = (CStatisticsListItem *)(StatisticsListByTime[i]);
    if( tempitem->m_Time == time ) {
			// found.
      item = (CStatisticsListItem *)(StatisticsListByTime[i]);
      break;
    } else if( tempitem->m_Time < time ) {
      break;
    }
  }
  */
  if (item==NULL) {
		// ask the count manager how many salves were running on that time
    long count = m_pMyLoadAgendaManager->m_SlaveCountManager.GetSlaveCount(time);
    if (count == 0) 
      return;
		// create the list item for that time collecting from all LGs
    item = new CStatisticsListItem(time, count);
    StatisticsListByTime.insert(item);
  }
  
	// insert the new statistics data to the item list
  if (item->NewStatistics(StatisticsMessage)) {
		// if it is the last statistics data for that time
    StatisticsListByTime.SetLastTimeSent(time);
		// send all statistics data until that time. if got all statistics transactions from
		// all LGs, earlier statistics data will not appear.
    while ((item = (CStatisticsListItem *)StatisticsListByTime.removeAt(0)) != NULL) {
      double itemTime = item->GetTime();
      CAssociativeArray Message;
      //
      // ToDo: this one will need to be removed in the integration with Alon.
      //Message.AddCouple("$STATISTICSMANAGERDATA",StatisticsMessage.GetValueOfParameter("$STATISTICSMANAGERDATA"));
      //
      // Set the statistics message & Time.
      Message.AddCouple("$STATISTICSMANAGERDATA",item->GetStatisticsString());
      Message.AddCouple("$TIME", itemTime);
      //
      // Send the statistics.
      m_pMyLoadAgendaManager->m_pMyLoadGenerator->SendMessage(Message,3);
      delete item;
      //
      // If time arrived, stop loop.
      if (itemTime == time)
        break;
    }
  }
}

