/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// LoadEngineVerificationClasses.cpp: implementation of the CLoadThreadTransactionManager class.
//
//////////////////////////////////////////////////////////////////////

#include "associativearray.h"
#include "loadgeneratorscript.h"
#include "loadengineverification.h"
#include "loadengineverificationclasses.h"

//////////////////////////////////////////////////////////////////////
// CLoadTransaction Class
//////////////////////////////////////////////////////////////////////

CLoadTransaction::CLoadTransaction(const char* name, IGeneralThreadTransactionManager *manager) :
  RVCollectableString(name),
  m_MyManager(manager),
  m_Ending(false)
{
}

CLoadTransaction::~CLoadTransaction()
{
  m_EventsList.clearAndDestroy();
}

static const char* StatusNamesArray[] = {
  "Successful",
  "Marked",
  "Failed"
};

const char* CLoadTransaction::GetStatusName(CLoadTransaction::Status status)
{
  return StatusNamesArray[status];
}

CLoadTransactionEvent* CLoadTransaction::RecordEvent(const char* name, const char* description, double time, WLLogMessage::Severity severity, const char* extraDataType, void* extraData, unsigned long len)
{
  if (m_Ending)
    return NULL;

  CLoadTransactionEvent *event = new CLoadTransactionEvent(name, description, severity, extraDataType, extraData, len);
  m_EventsList.insert(event);
  m_MyManager->InsertEvent(event);

	event->m_Time = time - m_MyManager->vGetBeginTime();
  event->m_LineNumber = 1;
  event->m_FileName = "None";
  return event;
}

const char* CLoadTransaction::toString()
{
  CAssociativeArray Array;
  Array.AddCouple("InstanceID", m_InstanceID);
  Array.AddCouple("ParentID", m_ParentID);
	Array.AddCouple("TopInstanceID", m_TopInstanceID); // Eyal 28.03.00
  Array.AddCouple("Level", m_Level);
  Array.AddCouple("TransactionName", m_TransactionName);
  Array.AddCouple("Reason", m_Reason);
  Array.AddCouple("Time", m_Time);
  Array.AddCouple("Duration", m_Duration);
  Array.AddCouple("LoadSize", m_LoadSize);
  Array.AddCouple("Severity", (long)m_Severity);
  Array.AddCouple("NumEvents", (long)m_EventsList.entries());
	Array.AddCouple("ExtraDataType", m_ExtraDataType); // Eyal 28.03.00
  Array.AddCouple("ExtraData", m_ExtraData.data()); // Eyal 28.03.00

  for (unsigned int i=0; i<m_EventsList.entries(); i++) {
    CLoadTransactionEvent *event = (CLoadTransactionEvent*)m_EventsList(i);
    Array.AddArrayCouple("Event", event->toString(), i+1);
  }
  m_ToString = Array.ToString();
  return m_ToString.data();
}

//////////////////////////////////////////////////////////////////////
// CLoadTransactionEvent Class
//////////////////////////////////////////////////////////////////////

CLoadTransactionEvent::CLoadTransactionEvent(const char* name, const char* description, WLLogMessage::Severity severity, const char* extraDataType, void* extraData, unsigned long len) :
  m_Name(name),
  m_Description(description),
  m_Severity(severity)
{
  if (extraDataType) {
    m_ExtraDataType = extraDataType;
    m_ExtraData = (char*)extraData;
  }
}

CLoadTransactionEvent::~CLoadTransactionEvent()
{
}

const char* CLoadTransactionEvent::toString()
{
  CAssociativeArray Array;
  Array.AddCouple("Name", m_Name);
  Array.AddCouple("Description", m_Description);
  Array.AddCouple("Severity", (long)m_Severity);
  Array.AddCouple("Time", m_Time);
  Array.AddCouple("LineNumber", m_LineNumber);
  Array.AddCouple("FileName", m_FileName);
  Array.AddCouple("ExtraDataType", m_ExtraDataType);
  Array.AddCouple("ExtraData", m_ExtraData);

  m_ToString = Array.ToString();
  return m_ToString.data();
}

void CLoadTransactionEvent::SetScriptData(double time, const char *fileName, int lineNumber)
{
  m_Time = time;
  m_FileName = fileName;
  m_LineNumber = lineNumber;
}


//////////////////////////////////////////////////////////////////////
// Debug Printing
//////////////////////////////////////////////////////////////////////

void PrintTransaction(const char* input)
{
  CAssociativeArray Array;
  Array.FromString(input);

  RVCString tab;
  for (int j=1;j<=Array.GetLongValueOfParameter("Level");j++)
    tab+="  ";

  cout << tab << "TransactionName: " << Array.GetValueOfParameter("TransactionName") << endl;
  cout << tab << "  Properties:" << endl;
  cout << tab << "    InstanceID: \t" << Array.GetValueOfParameter("InstanceID") << endl;
  cout << tab << "    ParentID: \t" << Array.GetValueOfParameter("ParentID") << endl;
  cout << tab << "    Level: \t" << Array.GetValueOfParameter("Level") << endl;
  cout << tab << "    Severity: \t" << WLLogMessage::Name((WLLogMessage::Severity)Array.GetLongValueOfParameter("Severity")) << endl;
  cout << tab << "    Reason: \t" << Array.GetValueOfParameter("Reason") << endl;
  cout << tab << "    Time: \t" << Array.GetValueOfParameter("Time") << endl;
  cout << tab << "    Duration: \t" << Array.GetValueOfParameter("Duration") << endl;
  cout << tab << "    LoadSize: \t" << Array.GetValueOfParameter("LoadSize") << endl;
  cout << tab << "    NumEvents: \t" << Array.GetValueOfParameter("NumEvents") << endl;
  cout << tab << "  Events:" << endl;

  for (int i=1; i<=Array.GetLongValueOfParameter("NumEvents"); i++) {
    CAssociativeArray EventArray;
    EventArray.FromString(Array.GetValueOfParameterArray("Event", i));
    cout << tab << "    Event:" << EventArray.GetValueOfParameter("Name") << endl;
    cout << tab << "      Description: \t" << EventArray.GetValueOfParameter("Description") << endl;
    cout << tab << "      Severity: \t" << WLLogMessage::Name((WLLogMessage::Severity)EventArray.GetLongValueOfParameter("Severity")) << endl;
    cout << tab << "      Time: \t" << EventArray.GetValueOfParameter("Time") << endl;
    cout << tab << "      LineNumber: \t" << EventArray.GetValueOfParameter("LineNumber") << endl;
    cout << tab << "      FileName: \t" << EventArray.GetValueOfParameter("FileName") << endl;
    cout << tab << "      ExtraDataType: \t" << EventArray.GetValueOfParameter("ExtraDataType") << endl;
    cout << tab << "      ExtraData: \t" << EventArray.GetValueOfParameter("ExtraData") << endl;
  }
}

void PrintTransactionTree(const char* input)
{
  CAssociativeArray Message;
  CAssociativeArray Array;
  Message.FromString(input);
  //*
  Array.FromString(Message.GetValueOfParameter("$TRANSACTIONTREE"));
  cout << "ThreadNum:\t" << Array.GetValueOfParameter("ThreadNum") << endl;
  cout << "RoundNum\t" << Array.GetValueOfParameter("RoundNum") << endl;
  cout << "NumTransactions\t" << Array.GetValueOfParameter("NumTransactions") << endl;
  cout << "Transactions:" << endl;
  for (int i=1; i<=Array.GetLongValueOfParameter("NumTransactions"); i++) {
    PrintTransaction(Array.GetValueOfParameterArray("Transaction", i));
  }
  /*/
  static CWLTransactionInstancesManager tm;
  static int count = 0;
  if (!count++) {
    tm.Init();
    tm.Connect("TransactionDB.mdb");
  }

  tm.Write(Message.GetValueOfParameter("$TRANSACTIONTREE"));
  //tm.Terminate();
  */
}

bool CLoadTransaction::End( WLLogMessage::Severity severity, double time, bool save, bool SendTimers) {
  RVCString Name;

//  m_Save = save;
	m_Duration = (time - m_MyManager->vGetBeginTime())- m_Time;
  m_Severity = severity;
  m_Ending = true;

	if(SendTimers) {
		Name = "";
		if (severity != WLLogMessage::Success) {
			m_Status = CLoadTransaction::Failed;
			if (m_Reason.length() == 0)
				m_Reason = "Not Specified";
			Name = Name + "Failed " + m_TransactionName + " Transactions : " + m_Reason;
			m_MyManager->AddLast(Name.data(), 1, CWLStatPointTypeEnum::TransactionReasonCounter);
		} else {
			if (m_Reason.length() == 0)
				m_Reason = " ";
			Name = Name + m_TransactionName + " Time";
			m_MyManager->AddLast(Name.data(), m_Duration, CWLStatPointTypeEnum::TransactionTimer);
		}

		Name = "";
		Name = Name + "Total "+ m_TransactionName + " Transactions";
		m_MyManager->AddLast(Name.data(), 1, CWLStatPointTypeEnum::TransactionCounter);
		Name = "";
		Name = Name + CLoadTransaction::GetStatusName(m_Status) + " " + m_TransactionName + " Transactions";
		m_MyManager->AddLast(Name.data(), 1, CWLStatPointTypeEnum::TransactionCounter);
		if (m_Status == Marked) {
			Name = "";
			Name = Name + CLoadTransaction::GetStatusName(Successful) + " " + m_TransactionName + " Transactions";
			m_MyManager->AddLast(Name.data(), 1, CWLStatPointTypeEnum::TransactionCounter);
		}

	}
  return true;
}

