/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// SlaveLoadSizeManager.cpp: implementation of the CSlaveLoadSizeManager class.
//
//////////////////////////////////////////////////////////////////////

#include "slaveloadsizemanager.h"
#include "tracedll.h"

class CLoadSizeListItem :  public RVCollectable
{
  friend class CSlaveLoadSizeManager;
private:
  double m_Time;
  double m_UpdateTime;
  double m_WeightFromStart;
  double m_WeightInSlice;
  long   m_LoadSize;
	  
public:
  CLoadSizeListItem(double time, double updateTime, double weight, long loadsize);
  ~CLoadSizeListItem();
  
  void Update(double time, long loadsize);
  void Close();

  double GetWeightUntilNow() { return m_WeightFromStart+m_WeightInSlice; }

  virtual bool isEqual(const RVCollectable* o) const;
  virtual int compareTo(const  RVCollectable* o) const;
  virtual char *nameOf() const;
};

CLoadSizeListItem::CLoadSizeListItem(double time, double updateTime, double weight, long loadsize)
{
  m_Time = time;
  m_UpdateTime = updateTime;
  m_LoadSize = loadsize;
  m_WeightFromStart = weight;
  m_WeightInSlice = 0;
}

CLoadSizeListItem::~CLoadSizeListItem()
{
}

void CLoadSizeListItem::Update(double time, long loadsize)
{
  m_WeightInSlice = m_WeightInSlice + (time-m_UpdateTime)*m_LoadSize;
  m_LoadSize = loadsize;
  m_UpdateTime = time;
}

void CLoadSizeListItem::Close()
{
  Update(m_Time, m_LoadSize);
}


bool CLoadSizeListItem::isEqual(const RVCollectable* o) const
{
  return( ((CLoadSizeListItem*)o)->m_Time == m_Time );
}

int CLoadSizeListItem::compareTo(const  RVCollectable* o) const
{
  if( ((CLoadSizeListItem *)o)->m_Time == m_Time )
    return 0;

  if( ((CLoadSizeListItem *)o)->m_Time < m_Time )
    return 1;		
  else
    return -1;
}

char *CLoadSizeListItem::nameOf() const
{
  return ("LoadSizeListItem");
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSlaveLoadSizeManager::CSlaveLoadSizeManager()
{
  m_BeginTime = 0;
  m_SliceSize = 0;
  m_CurrentLoadSize = 0;
}

CSlaveLoadSizeManager::~CSlaveLoadSizeManager()
{
  m_LoadSizeList.clearAndDestroy();
}

void CSlaveLoadSizeManager::Start( double beginTime, double sliceSize )
{
  m_BeginTime = beginTime;
  m_SliceSize = sliceSize;
  m_NextSliceEnd = sliceSize;
  m_LoadSizeList.insert(new CLoadSizeListItem(m_SliceSize, 0, 0, 0));
}

void CSlaveLoadSizeManager::NewLoadSize( long delta, double now )
{
  //
  // make sure that Start was called (and than m_BeginTime>0)
  WlAssert (m_BeginTime>0);

	// protect load size list when inserting a new one item
	RVMutexLock::LockGuard lock(m_ListLock);

  double timeFromStart = now - m_BeginTime;
  m_CurrentLoadSize += delta;
  
  CLoadSizeListItem *lastitem = (CLoadSizeListItem *)m_LoadSizeList.last();
  //
  // if it's the same time or a little after, enlarge the time a little.
  if (m_NextSliceEnd <= timeFromStart) {
    do {
      lastitem->Close(); 
      m_NextSliceEnd += m_SliceSize;
      CLoadSizeListItem *item = new CLoadSizeListItem(m_NextSliceEnd, lastitem->m_Time, lastitem->GetWeightUntilNow(), lastitem->m_LoadSize);
      m_LoadSizeList.insert(item);
      lastitem = item;
    } while (m_NextSliceEnd<=timeFromStart);
  } 
  lastitem->Update(timeFromStart, m_CurrentLoadSize);
	WlTraceMsg("CSlaveLoadSizeManager::NewLoadSize(CurrentLoadSize=%d)", m_CurrentLoadSize);
}

void CSlaveLoadSizeManager::AddThread(double now)
{
  NewLoadSize(1, now);
}

void CSlaveLoadSizeManager::RemoveThread(double now)
{
  NewLoadSize(-1, now);
}

void CSlaveLoadSizeManager::GetLoadSize( double endTime, double& averageLoadSize, double& sliceLoadSize)
{
  // protect load size list while reading
	RVMutexLock::LockGuard lock(m_ListLock);

  CLoadSizeListItem *item = (CLoadSizeListItem *)m_LoadSizeList[0];
  while (item->m_Time < endTime && m_LoadSizeList.entries()>1) {
    delete m_LoadSizeList.removeAt(0);
    item = (CLoadSizeListItem *)m_LoadSizeList[0];
  }
  averageLoadSize = (item->m_WeightFromStart + item->m_WeightInSlice + (endTime - item->m_UpdateTime)*item->m_LoadSize) / endTime;
  if (item->m_Time<endTime)
    sliceLoadSize = item->m_LoadSize;
  else
    sliceLoadSize = (item->m_WeightInSlice + (endTime - item->m_UpdateTime)*item->m_LoadSize) / m_SliceSize;
	WlTraceMsg("CSlaveLoadSizeManager::GetLoadSize(averageLoadSize=%d, sliceLoadSize=%d)", averageLoadSize, sliceLoadSize);
}

