/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// File: ThreadGlobalsHandler.cpp 
//
//////////////////////////////////////////////////////////////////////

#include "threadglobalshandler.h"
#include "loadthreadsmanager.h"
#include "loadgenerator.h"
#include "globalvariableproperties.h"
#include "tracedll.h"
#include "loadengineglobals.h"
#include "loadengineinifileproperties.h"

//////////////////////////////////////////////////////////////////////
// Implementation of the CThreadsManagerGlobalsRequestHandler class.
//
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CThreadsManagerGlobalsRequestHandler::
CThreadsManagerGlobalsRequestHandler(CLoadThreadsManager *threadsManager)
{
	m_Active  = true;
	m_Message = NULL;
	m_ThreadsManager = threadsManager;
	m_pMyLoadGeneratorSlave = m_ThreadsManager->GetLoadGeneratorSlave();
}

CThreadsManagerGlobalsRequestHandler::~CThreadsManagerGlobalsRequestHandler()
{
	m_Active = false;
	m_GetEvent.Set();
}

//////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////

void CThreadsManagerGlobalsRequestHandler::
	Globals_Set(const char* variableName, CGlobalVariableProperties *properties, Level level)
{
	CAssociativeArray request;

	AddGlobalVriablesObjectName(request, level);

	request.AddCouple("$METHOD", "SET");
	request.AddCouple("$VARNAME", variableName);
	request.AddCouple("$TYPE", properties->GetType());
	request.AddCouple("$VALUE", properties->GetUnformatedCharValue());

	SendRequest(request, level);
}

void CThreadsManagerGlobalsRequestHandler::Globals_Add(const char* variableName, CGlobalVariableProperties *properties, Level level)
{
	CAssociativeArray request;

	AddGlobalVriablesObjectName(request, level);

	request.AddCouple("$METHOD", "ADD");
	request.AddCouple("$VARNAME", variableName);
	request.AddCouple("$TYPE", properties->GetType());
	request.AddCouple("$VALUE", properties->GetUnformatedCharValue());

	SendRequest(request, level);
}

CGlobalVariableProperties* CThreadsManagerGlobalsRequestHandler::Globals_SyncAdd(const char* variableName, CGlobalVariableProperties *properties, Level level)
{
	m_Message = NULL;
	CAssociativeArray request;

	AddGlobalVriablesObjectName(request, level);

	request.AddCouple("$METHOD", "SADD");
	request.AddCouple("$VARNAME", variableName);
	request.AddCouple("$TYPE", properties->GetType());
	request.AddCouple("$VALUE", properties->GetUnformatedCharValue());
	SendRequest(request, level);

	if(!WaitMessage()) return NULL;

	return m_Message;
}

CGlobalVariableProperties *CThreadsManagerGlobalsRequestHandler::Globals_Get(const char* variableName, Level level)
{
	m_Message = NULL;
	CAssociativeArray request;

	AddGlobalVriablesObjectName(request, level);

	request.AddCouple("$METHOD", "GET");
	request.AddCouple("$VARNAME", variableName);
	SendRequest(request, level);

	if(!WaitMessage()) return NULL;

	return m_Message;
}

inline bool CThreadsManagerGlobalsRequestHandler::WaitMessage(void)
{
	CAssociativeArray *message = NULL;

	while(message = m_pMyLoadGeneratorSlave->GetMessage()) {
		if(!m_Active)
			return false;

		if(message->KeyExists("$GLOBALSNAME")) {
			HandleReplay(*message);
			delete message;
			m_GetEvent.Reset();
			return true;
		}
		m_pMyLoadGeneratorSlave->PutMessage(message);
		return false;
	}
	return false;
}

inline bool CThreadsManagerGlobalsRequestHandler::IsAbort(void)
{
	return m_ThreadsManager->IsTerminate();
}

void CThreadsManagerGlobalsRequestHandler::ReleaseObject(void)
{
	m_Active = false;
	m_GetEvent.Set();
}

void CThreadsManagerGlobalsRequestHandler::HandleReplay(CAssociativeArray &replay)
{
	m_Message = 
		new CGlobalVariableProperties(replay.GetValueOfParameter("$TYPE"),
																	replay.GetValueOfParameter("$VALUE"));
	
	m_GetEvent.Set();
}

void CThreadsManagerGlobalsRequestHandler::SendRequest(CAssociativeArray &request, Level level)
{
	const char* address;

	switch(level) {
		case LGLevelPerAgendaInMix: // level 1
		case LGLevelPerAgenda:      // level 2
			m_pMyLoadGeneratorSlave->SendMessage(request, 4);
			return;

		case MetaLevelPerAgenda:    // level 3
		case MetaLevel:             // level 4
			address = m_pMyLoadGeneratorSlave->m_MonitorGlobalVariablesAddress.data();
			m_pMyLoadGeneratorSlave->SendMessageToAddress(request, address, 4);
			return;

		default:
			break;
	}
}

void CThreadsManagerGlobalsRequestHandler::AddGlobalVriablesObjectName(CAssociativeArray &request, Level level)
{

  // Chaim - The GLOBALSNAME is used as the key's object's name. Since the KGLevel and
  // the Meta level stuff are kept in different places (LG vs. TT) it's ok to use the agenda path
  // for LGLevelPerAgendaInMix and MetaLevelPerAgenda. The GLOBALSREALNAME is used to check
  // the agenda is being aborted.
	switch(level) {
		case LGLevelPerAgendaInMix: {
			request.AddCouple("$GLOBALSNAME", m_ThreadsManager->GetAgendaPath());
			request.AddCouple("$GLOBALSREALNAME", m_ThreadsManager->GetAgendaName());
			return;
		}
		case LGLevelPerAgenda: {
			request.AddCouple("$GLOBALSNAME", LOAD_GENERATOR_GLOBALS_NAME);
			request.AddCouple("$GLOBALSREALNAME", m_ThreadsManager->GetAgendaName());
			return;
		}
		case MetaLevelPerAgenda: {
			request.AddCouple("$GLOBALSNAME", m_ThreadsManager->GetAgendaPath());
			request.AddCouple("$GLOBALSREALNAME", m_ThreadsManager->GetAgendaName());
			return;
		}
		case MetaLevel: {
			request.AddCouple("$GLOBALSNAME", MONITOR_GLOBALS_NAME);
			request.AddCouple("$GLOBALSREALNAME", m_ThreadsManager->GetAgendaName());
			return;
		}
		default:
			break;
	}
}


//////////////////////////////////////////////////////////////////////
// Implementation of the CThreadGlobalsRequestHandler class.
//
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CThreadGlobalsRequestHandler::
CThreadGlobalsRequestHandler(CLoadThreadInstance *threadInstance):
CThreadsManagerGlobalsRequestHandler(threadInstance->GetThreadsManager())
{
	m_ThreadInstance = threadInstance;
}

//////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////

inline bool CThreadGlobalsRequestHandler::WaitMessage(void)
{
	while(!m_Message) {
		if(IsAbort() || !m_Active)
			return false;
		m_GetEvent.Wait(CLEIniFileProperties::GlobalsHandlerWaitTime);
	}
	m_GetEvent.Reset();
	return true;
}

inline bool CThreadGlobalsRequestHandler::IsAbort(void)
{
	return m_ThreadInstance->isAbort(false);
}

void CThreadGlobalsRequestHandler::AddGlobalVriablesObjectName(CAssociativeArray &request, Level level)
{
	request.AddCouple("$THREADID", (long)m_ThreadInstance->GetSlaveThreadNum());

	switch(level) {
		case LGLevelPerAgendaInMix: {
			request.AddCouple("$GLOBALSNAME", (m_ThreadInstance->GetCurrentScript())->GetScriptFileName());
			request.AddCouple("$GLOBALSREALNAME", (m_ThreadInstance->GetCurrentScript())->GetScriptName());
			return;
		}
		case LGLevelPerAgenda: {
			request.AddCouple("$GLOBALSNAME", LOAD_GENERATOR_GLOBALS_NAME);
			request.AddCouple("$GLOBALSREALNAME", (m_ThreadInstance->GetCurrentScript())->GetScriptName());
			return;
		}
		case MetaLevelPerAgenda: {
			request.AddCouple("$GLOBALSNAME", m_ThreadsManager->GetAgendaPath());
			request.AddCouple("$GLOBALSREALNAME", m_ThreadsManager->GetAgendaName());
			return;
		}
		case MetaLevel: {
			request.AddCouple("$GLOBALSNAME", MONITOR_GLOBALS_NAME);
			request.AddCouple("$GLOBALSREALNAME", m_ThreadsManager->GetAgendaName());
			return;
		}
		default:
			break;
	}
}

