/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// RFS.cpp: implementation of the RFS class.
//
//////////////////////////////////////////////////////////////////////

#include "rfs.h"
#include "rfsevent.h"
#include "rfsdataobject.h"
#include "netkernelmessagehandler.h"

#include <collstr.h>


const char* RFS::SECURITY_PATH_TOKEN = "rvtemp";

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

RFS::RFS()
{
	mIsConnectionOwner = true;
	mUniqueAndSecuredPathToken = NULL;

	mConnection = new CNetMessageHandler("RFSAgent");
	mConnection->Init();

	mConnection->AddMessagesListener(this);

	mGlobalRetVal      = new RVCString();
	mRFSOperationEvent = new RFSEvent();
	mOperationSection  = new RVCriticalSection();
}

RFS::RFS(CNetMessageHandler* connection)
{
	mIsConnectionOwner = false;
	mUniqueAndSecuredPathToken = NULL;

	mConnection = connection;
	mConnection->AddMessagesListener(this);

	mGlobalRetVal = new RVCString();
	mRFSOperationEvent = new RFSEvent();
	mOperationSection  = new RVCriticalSection();
}

RFS::~RFS()
{
	mConnection->RemoveMessagesListener(this);

	delete mRFSOperationEvent;
	delete mGlobalRetVal;
	delete mOperationSection;

	if( mUniqueAndSecuredPathToken )
	{
		delete mUniqueAndSecuredPathToken;
	}

	if( mIsConnectionOwner && mConnection )
	{
		mConnection->Terminate();
		delete mConnection;
	}

	mConnection = NULL;
}


//////////////////////////////////////////////////////////////////////
// Methods Implementation
//////////////////////////////////////////////////////////////////////

bool RFS::SendArchive(const char *remoteHost, FilesAssociationArchive *archive)
{
	try
	{
		RVCString result;
		RFSDataObject* retDataObj = NULL;
		RVCString exceptionMessage("Could not copy file:\n");
		RVCollectableString* srcFile;
		RVCollectableString* dstFile;
		RVHashDictionaryIterator& iterator = archive->GetIterator();
		CNetMessageHandler::CopyFileCompletionState copyStatus;

		mRFSOperationEvent->Reset();

		while( iterator() )
		{
			srcFile = (RVCollectableString*)iterator.key();
			dstFile = (RVCollectableString*)iterator.value();

			retDataObj = CopyFile(remoteHost, srcFile->data(), dstFile->data());

			if( retDataObj == NULL )
			{
				return false;
			}

			copyStatus = CNetMessageHandler::CopyFileCompletionState(retDataObj->mN0);

			if(copyStatus != CNetMessageHandler::CopyFileSucceed)
			{
				return false;
			}
		}
	}
	catch(...)
	{
		return false;
	}

	return true;
}

bool RFS::GetArchive(const char *remoteHost, FilesAssociationArchive *archive)
{
	CAssociativeArray message;

	message.AddCouple("$ARCHIVE", archive->ToString());

	RFSDataObject* result = (RFSDataObject*)SendRFSRequest(remoteHost, "$GETARCHIVE", &message);

	if( result )
	{
		if( atoi(result->mS0.data()) == 0 )
		{
			//throw RFSException(result->mS2.data());
			return false;
		}

		//*mGlobalRetVal = result->mS1;
	}
	else
	{
		return false;
		//throw RFSException("Unkown Exception: GetArchive");
	}

	//return mGlobalRetVal->data();
	return true;
}

RFSString RFS::CreateRemoteBaseDirectory(const char* remoteHost, DirectorySym dirSym, bool deleteOldBaseDir)
{
	RVCString dirPath;
	RFSString baseDir;

	switch(dirSym)
	{
		case InstallationDir:
			baseDir = GetRemoteInstallationDir(remoteHost);
		break;

		case TempDir:
			baseDir = GetRemoteTempDirectory(remoteHost);
		break;

		case UserDir:
			baseDir = GetRemoteUserDir(remoteHost);
		break;

		default:
			baseDir = GetRemoteInstallationDir(remoteHost);
	}

	if( !baseDir )
	{
		return RFS_NULL;
	}
	else
	{
		dirPath = baseDir;
	}

	dirPath += "\\";
	dirPath += SECURITY_PATH_TOKEN;

	CreateRemoteDirectory(remoteHost, dirPath);

	dirPath += "\\";
	dirPath += mConnection->GetLocalHostName();

	if( !CreateRemoteDirectory(remoteHost, dirPath, deleteOldBaseDir) )
	{
		return RFS_NULL;
	}

	*mGlobalRetVal = dirPath;

	return mGlobalRetVal->data();
}

RFSString RFS::CreateRemoteDirectory(const char *remoteHost, const char *dirPath, bool deleteOldDir)
{
	if( CheckRemotePathAccessibility(dirPath) )
	{
		CAssociativeArray message;

		message.AddCouple("$DIRPATH", dirPath);
		message.AddCouple("$DELOLDDIR", long(deleteOldDir));

		RVCString* retVal = ((RVCString*)SendRFSRequest(remoteHost, "$CREATEDIR", &message));

		if( retVal && atoi(retVal->data()) )
		{
			return dirPath;
		}
	}

	return RFS_NULL;
}

RFSString RFS::DeleteRemoteFile(const char *remoteHost, const char *dirPath)
{
	return RemoveRemoteFile(remoteHost, dirPath, false);
}

RFSString RFS::DeleteRemoteDirectory(const char *remoteHost, const char *dirPath)
{
	return RemoveRemoteFile(remoteHost, dirPath, true);
}

const char* RFS::RemoveRemoteFile(const char *remoteHost, const char *fileName, bool isDirectory)
{
	if( CheckRemotePathAccessibility(fileName) )
	{
		CAssociativeArray message;

		message.AddCouple("$FILEPATH", fileName);
		message.AddCouple("$ISDIR", long(isDirectory));

		RVCString* retVal = ((RVCString*)SendRFSRequest(remoteHost, "$DELETEFILE", &message));

		if( retVal && atoi(retVal->data()) )
		{
			return fileName;
		}
	}

	return RFS_NULL;
}

RFSString RFS::GetSecurityPathToken()
{
	return SECURITY_PATH_TOKEN;
}

RFSString RFS::GetRemoteInstallationDir(const char* remoteHost)
{
	return GetRemoteDirectory(remoteHost, "$GETTVTOP");
}

RFSString RFS::GetRemoteTempDirectory(const char *remoteHost)
{
	return GetRemoteDirectory(remoteHost, "$GETTMPDIR");
}

RFSString RFS::GetRemoteUserDir(const char *remoteHost)
{
	return GetRemoteDirectory(remoteHost, "$GETUSRDIR");
}

const char* RFS::GetRemoteDirectory(const char* remoteHost, const char* dirSymbol)
{
	CAssociativeArray message;

	message.AddCouple(dirSymbol, "");

	RVCString* dirPath = ((RVCString*)SendRFSRequest(remoteHost, "$GETDIRPATH", &message));

	if( dirPath && *dirPath != "null" )
	{
		return dirPath->data();
	}

	return RFS_NULL;
}

void* RFS::SendRFSRequest(const char* remoteHost, const char* requestSymbol, CAssociativeArray* sendMessage)
{
	RVCString error(RVSize_T(40));
	RVCString address(RVSize_T(40));
  CAssociativeArray* message = sendMessage;

	if( message == NULL )
	{
		message = new CAssociativeArray();
	}

	address += "theTml@";
	address += remoteHost;

	error += "RFS request error, symbol: ";
	error += requestSymbol;

	mRFSOperationEvent->Reset();
	message->AddCouple("$RFSRequest", requestSymbol);
	mConnection->SendMessage(*message, address, error);

	if( sendMessage == NULL )
	{
		delete message;
	}

	if( !mConnection->IsActive() ) //@@Eyal: remove
	{
	//	MessageBox(NULL, "mConnection not active", "Error", MB_OK);
		return NULL;
	}

	if( mRFSOperationEvent->Wait(120000) == RV_THR_TIMEOUT )
	{
		return NULL;
	}

	mRFSOperationEvent->Reset();

	return mRFSOperationEvent->GetResult();	
}

void RFS::SendRFSResponse(const char* remoteHost, const char* responseSymbol, CAssociativeArray* sendMessage)
{
	//
	// Not implemented
}

void RFS::ProcessRFSRequest(CAssociativeArray& message, RVCString& symbol)
{
	//
	// Not implemented
}

void RFS::ProcessRFSResponse(CAssociativeArray& message, RVCString& symbol)
{
	if( message.KeyExists("$RETDIRPATH") )
	{
		RVCString* result = new RVCString(message.GetValueOfParameter("$RETDIRPATH"));
		mRFSOperationEvent->SetResultAndSignal(result);
	}
	if( message.KeyExists("$CREATEDIRSTATUS") )
	{
		RVCString* result = new RVCString(message.GetValueOfParameter("$CREATEDIRSTATUS"));
		mRFSOperationEvent->SetResultAndSignal(result);
	}
	if( message.KeyExists("$DELETEFILESTATUS") )
	{
		RVCString* result = new RVCString(message.GetValueOfParameter("$DELETEFILESTATUS"));
		mRFSOperationEvent->SetResultAndSignal(result);
	}
	if( message.KeyExists("$GETARCHIVESTATUS") )
	{
		printf("ST=%s\n", message.GetValueOfParameter("$GETARCHIVESTATUS"));

		RFSDataObject* result = new RFSDataObject();

		result->mS0 = message.GetValueOfParameter("$GETARCHIVESTATUS");
		result->mS1 = message.GetValueOfParameter("$RESULTPATH");
		result->mS2 = message.GetValueOfParameter("$EXCEPTIONMSG");

		mRFSOperationEvent->SetResultAndSignal(result);
	}
}

void RFS::CopyFileDoneDispatched(int copyFileCompletionState,
														  	 const char* copyStatusFileName,
															   const char* localFileName, 
															   const char* remoteFileName, 
															   const char* remoteHostAddress,
  														   CAssociativeArray& message)
{
	RFSDataObject* dataObj = new RFSDataObject();

	dataObj->mN0 = copyFileCompletionState;
	dataObj->mS0 = copyStatusFileName;
	dataObj->mS1 = localFileName;
	dataObj->mS2 = remoteFileName;
	dataObj->mS3 = remoteHostAddress;

	dataObj->mMessage.FromString(message.ToString());

	mRFSOperationEvent->SetResultAndSignal(dataObj);

	printf("%s\n%s\n%s\n%s\n", copyStatusFileName, localFileName, remoteFileName, remoteHostAddress);
}

RFSDataObject* RFS::CopyFile(const char* remoteHost, const char *srcFile, const char *dstFile)
{
	mConnection->CopyFile(srcFile, dstFile, remoteHost);

	if( mRFSOperationEvent->Wait(100000) == RV_THR_TIMEOUT )
	{
		mRFSOperationEvent->Reset();
		return NULL;
	}

	mRFSOperationEvent->Reset();

	return (RFSDataObject*)mRFSOperationEvent->GetResult();
}

bool RFS::CheckRemotePathAccessibility(const char* dirPath)
{
	RVCString path(dirPath);

	if( path.contains(SECURITY_PATH_TOKEN, RVCString::ignoreCase) )
	{
		return true;
	}

	return false;
}

bool RFS::IsNull(RFSString& str)
{
#ifdef SIMPLE_STRING
	return str == NULL;
#else
	return str.length() == 0;
#endif
}

RFSString RFS::GetLastErrorMessage()
{
	return "";
}


//////////////////////////////////////////////////////////////////////
// Implementation of the RFSException class.
//
//////////////////////////////////////////////////////////////////////

RFSException::RFSException()
{
}

RFSException::RFSException(RFSException& exception)
{
	mMessage = exception.GetMessage();
}

RFSException::RFSException(const char* message)
{
	mMessage = message;
}

RFSException::~RFSException()
{
}

/*
inline const char* RFSException::GetMessage()
{
	return mMessage.data();
}
*/
