/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// TestTalkRequester.cpp: implementation of the CTestTalkNetRequester class.
//
//////////////////////////////////////////////////////////////////////

#include "testtalknetrequester.h"
#include "supportinformationgenerator.h"


static unsigned long TESTTALK_REQUESTER_ID = 0;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTestTalkNetRequester::CTestTalkNetRequester(void)
{
	clientID = ++TESTTALK_REQUESTER_ID;	
}

CTestTalkNetRequester::~CTestTalkNetRequester()
{
	NETTerminate();
}


//////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////

// Init - Initialize net configurations
// @param name - testtalk client name
bool CTestTalkNetRequester::Init(const char* name)
{
	char charID[20];
  sprintf(charID, "%d", clientID);
  
  RVCString netName = name;
	netName += charID;

	return NETInit(netName.data());
}

// Terminate any net connections
void CTestTalkNetRequester::Terminate(void)
{
	NETTerminate();
}

// Ping - Sends any ping message to remote testtalk,
//				return true if sending was successfull false else
// @param data     - data to send
// @param dataSize - data length
// @param hostName - host to send the data to
bool CTestTalkNetRequester::SendPing(const char* data, 
																		 size_t dataSize, 
																	   const char* hostName)
{
	RVCString address = "theTml@";
	address += hostName;
	address += ":9000";

	return NETSendMessage(data, dataSize, address);
}

inline char* CTestTalkNetRequester::WaitForPingReply(void)
{
	return (char*)NETWaitMessage();
}

// TestTalkPing - Send simple ping to the specified host.
// The method will return an information object if the remote
// testtalk is alive and null else.
// Valid fields in the returned info object are: hostName and pingDuration.
CTestTalkNetRequester::CInformationObject*
CTestTalkNetRequester::TestTalkPing(const char* hostName)
{
	double  duration;
	clock_t start, finish;
	
	start = clock();
	bool connectionStatus = SendPing("Ping Message", strlen("Ping Message") + 1, hostName);
	finish = clock();

	if(connectionStatus) {
		duration = (double)(finish - start); 

		CInformationObject* info = new CInformationObject();
		info->hostName = hostName;
		info->pingDuration = duration;

		return info;
	}
	return NULL;
}

// Sends a ping with request for geting information and
// waits until the information arrives.
// Returns an information object if the remote testtalk is alive
// and the request has succeeded and null else.
// Valid fields in the returned info object are: hostName and pingDuration in all cases.
// More Valid fields: If type = Version - wlVersion
//										If type = SystemInfo - SystemInfo, IPConfig, WebLoad_ini, Tvcti_dll
//										If type = All - all fields are valid.
CTestTalkNetRequester::CInformationObject* 
CTestTalkNetRequester::TestTalkInformationPing(const char* hostName, 
																						CTestTalkNetRequester::InformationType type)
{
	char*  data;
	size_t dataSize;
	char*  replyMessage;
	CAssociativeArray pingMessage, pingReply;
	double  duration;
	clock_t start, finish;

	CreatePingMessage(hostName, type, pingMessage); 
	pingMessage.ToString(data, dataSize);
	
	start = clock();

	if( SendPing(data, dataSize, hostName) ) {
		replyMessage = WaitForPingReply();
	}
	else {
		return NULL;
	}

	finish = clock();
	duration = (double)(finish - start); 

	pingReply.FromString(replyMessage);
	CInformationObject* infoObject = CreatePingReplyInfoObject(pingReply);

	infoObject->pingDuration = duration;
	
	return infoObject;
}

bool CTestTalkNetRequester::CreatePingMessage(/*[IN]*/const char* hostName,
																							/*[IN]*/CTestTalkNetRequester::InformationType type,
																						  /*[OUT]*/CAssociativeArray& pingMessage)
{
	switch(type)
	{
		case WLVersion:
			pingMessage.AddCouple("$TTVERSION", "");
			break;
		case SystemInfo:
			pingMessage.AddCouple("$TTSYSTEMINFO", "");
			break;
		case All:
			pingMessage.AddCouple("$TTVERSION", "");
			pingMessage.AddCouple("$TTSYSTEMINFO", "");
			break;

		default:
			return false;
	}
	pingMessage.AddCouple("$TTPING", hostName);
	pingMessage.AddCouple("$FROM", NETWhoAmI());

	return true;
}

CTestTalkNetRequester::CInformationObject*
CTestTalkNetRequester::CreatePingReplyInfoObject(/*[IN]*/CAssociativeArray& pingReply)
{
	CInformationObject* infoObject = new CInformationObject();

	infoObject->pingDuration = 0; // default value
	infoObject->hostName		 = pingReply.GetValueOfParameter("$TTPINGREPLY");

	if(pingReply.KeyExists("$TTVERSION"))
		infoObject->wlVersion = pingReply.GetValueOfParameter("$TTVERSION");
	
	if(pingReply.KeyExists("$TTSYSTEMINFO")) {
		CAssociativeArray infoDB;
		  
		infoDB.FromString(pingReply.GetValueOfParameter("$TTSYSTEMINFO"));
		  
		infoObject->ipConfig    = infoDB.GetValueOfParameter("$IPCONFIG");
		infoObject->systemInfo  = infoDB.GetValueOfParameter("$SYSTEMREPORT");
		infoObject->Tvcti_dll   = infoDB.GetValueOfParameter("$TVCTIDLL");
		infoObject->WebLoad_ini = infoDB.GetValueOfParameter("$WEBLOADINI");
	}

	return infoObject;
}

bool CTestTalkNetRequester::GenerateReportFile(/*[OUT]*/ofstream& reportFile, 
		  																			   /*[IN]*/CTestTalkNetRequester::CInformationObject* infoObject)
{
	const char *string;
	const char *line = "\n===============================================================================\n";

	if(!infoObject || infoObject->hostName.length() == 0) {
		return false;
	}

	try {
		string = infoObject->hostName;
		reportFile<<line<<"Complete Report For Host: "<<string<<endl<<line;

		string = infoObject->ipConfig;
		reportFile<<line<<"IP Configuration: "<<line<<endl<<string;

		string = infoObject->systemInfo;
		reportFile<<line<<"System And Hardware Configuration: "<<line<<endl<<string;

		string = infoObject->Tvcti_dll;
		reportFile<<line<<"Webload Product Information(tvcti.dll): "<<line<<endl<<string;

		string = infoObject->WebLoad_ini;
		reportFile<<line<<"Webload.ini File: "<<line<<endl<<string;
	}
	catch(...) {
		return false;
	}
	return true;
}

