/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "basesocket.h"

//-********************************************************************

//Socket::Socket() is the default constructor for a socket.  You can set the
//  individual members of the Socket later.
Socket::Socket()
   : nFamily(PF_UNSPEC), nType(0), nProtocol(0),
     bBound(0), bSaveSocket(0), m_Socket(INVALID_SOCKET)
{
   nLastError = 0;
   //
   // set the default style for the socket.
   SetSocketStyle();
}


//Socket(SOCKET&) is a constructor based on a Winsock SOCKET descriptor.
Socket::Socket(SOCKET& newS) :
     bSaveSocket(0), m_Socket(newS)
{
   SOCKET_LEN nTemp;
   GetMyAddress(mySocketAddress, nTemp, m_Socket);   //Fills in our address information.
   nFamily=mySocketAddress.sin_family;
   GetPeerAddress(myPeerSocketAddress, nTemp, m_Socket); //Fills in out Peer's address info if it exists.
   nLastError = 0;
}


//This Socket destructor will close the socket it if has not be closed already.
//It will also delete the friend notification window as well.
Socket::~Socket(){
  if(m_Socket!=INVALID_SOCKET && !bSaveSocket){
     SetLingerOption(0, 60); //Don't wait till data gets sent.  Just kill the socket now.
     CloseSocket();
  }
}


Socket& Socket::operator=(Socket& newSocket){
//Does a deep copy of the Socket, as much as possible.
   nFamily                 = newSocket.nFamily;
   nType                   = newSocket.nType;
   nProtocol               = newSocket.nProtocol;
   nLastError              = newSocket.nLastError;
   m_Socket                       = newSocket.m_Socket;
   mySocketAddress         = newSocket.mySocketAddress;
   myPeerSocketAddress     = newSocket.myPeerSocketAddress;
   bSaveSocket             = newSocket.bSaveSocket;

   return *this;
}



int operator==(const Socket& socket1, const Socket& socket2){
//While it is possible that two sockets could refer to the same SOCKET (though
//  this would likely create a mess if not governed with care), it is defined as
//  not possible that two Sockets could lhave the same window member.  THis is
//  because the window is created uniquely on construction for each Socket.
   if(socket1.m_Socket != socket2.m_Socket)
      return 0;
   return 1;
}




//The SetSocketStyle function can be used to set or change some Socket member data.
//Note that the nNewFamily is also represented in the sockaddr_in member, thus they
//  should match.
void Socket::SetSocketStyle(int nNewFamily, int nNewType, int nNewProtocol){
   nFamily=nNewFamily;
   nType=nNewType;
   nProtocol=nNewProtocol;
}



//The CreateSocket function is much like the Winsock socket() function.
//This function assumes that nFamily, nType, and nProtocol are already set properly.
//Note also that since the return of socket() is assigned to 'm_Socket', that 'm_Socket' must not
//  be alredy used.  This is another way of saying that there can only be one
//  SOCKET for each Socket object.
int Socket::CreateSocket(){
   m_Socket=socket(nFamily, nType, nProtocol);
   if(m_Socket==INVALID_SOCKET){
      nLastError=CSocketLib::GetLastError();    //sockets call
      return WINSOCK_ERROR;
   }
   BOOL opt = TRUE;
   setsockopt(m_Socket, SOL_SOCKET, TCP_NODELAY, (char*)&opt, sizeof(opt));
   return WINSOCK_NOERROR;
}



//The CloseSocket() function is much like the Winsock closesocket() function.
int Socket::CloseSocket(){
   int nErrorValue;
   SOCKET s;

   if(m_Socket!=INVALID_SOCKET){
     s = m_Socket;
     m_Socket=INVALID_SOCKET; 
#ifndef __TV_UNIX__
     nErrorValue = closesocket(s);   //sockets call
#else
     (void)shutdown(s,SHUT_WR);
     nErrorValue = close(s);   //sockets call
#endif 
     if(nErrorValue){
         nLastError = CSocketLib::GetLastError();  //sockets call
         return WINSOCK_ERROR;
      }
   }
   bBound = 0;
   return WINSOCK_NOERROR;
}

//The ShutDownSocket() function is much like the Winsock shutdown() function.
//It is important to note that shutting down a socket essentially means that
//  you can't un-shut it down.  It is like a graceful way of preparing to
//  end a session.  It is somewhat like a yellow stoplight.
//You would want to use this function to close your socket, while still allowing data
//  be received from the network.  This is as opposed to CloseSocket(), which kills
//  all transfers in both directions.
//nShutMode is one of the enumerations: nShutModeNoRecv, nShutModeNoSend, or nShutModeNoRecvSend.
int Socket::ShutDownSocket(int nShutMode){
   int nErrorValue;

   if(m_Socket!=INVALID_SOCKET){
      nErrorValue = shutdown(m_Socket, nShutMode);  //sockets call
      if(nErrorValue){
         nLastError = CSocketLib::GetLastError();  //sockets call
         return WINSOCK_ERROR;
      }
      m_Socket=INVALID_SOCKET; //It's invalid now.
      bBound = 0;
   }
   return WINSOCK_NOERROR;
}

//BindSocket is much like the Winsock bind() function.
//Regardless of what mySocketAddress may have been previously, a call to 'bind()'
//  immediately makes the socket's address the one put into the bind() call.
//  Thus we simply always assign mySocketAddress to be boundSocketAddress.
//The address argument must be in network byte ordering. On the other hand, the
//  sockaddr_in class always keeps is addresses in network byte ordering.
int Socket::BindSocket(const char* hostname, u_short port){
  if(!CSocketLib::GetAddress(hostname, port, mySocketAddress) &&
      bind(m_Socket, (sockaddr*)&mySocketAddress, sizeof(sockaddr))) //bind() ideally returns 0.
  {
    nLastError=CSocketLib::GetLastError();  
    return WINSOCK_ERROR;
  }
  bBound = 1;
  return WINSOCK_NOERROR;
}



//This BindSocket simply binds with our previously defined member data socket address.
int Socket::BindSocket(u_short port)
{
	// @@@@Eyal - new code
	char localHostName[1024]; 
	
	if( CSocketLib::GetLocalHostName(localHostName, 1024, false) ) {
		CSocketLib::GetAddress(localHostName, port, mySocketAddress);
	}
	else {
		mySocketAddress.sin_family = AF_INET;
		mySocketAddress.sin_addr.s_addr = INADDR_ANY;
		mySocketAddress.sin_port = htons(port);
	}
  
  if(bind(m_Socket, (sockaddr*)&mySocketAddress, sizeof(sockaddr))) //bind() ideally returns 0.
  {
    nLastError=CSocketLib::GetLastError();  
    return WINSOCK_ERROR;
  }
  bBound = 1;
  return WINSOCK_NOERROR;
}

//This function stores the address into the reference argument 'socketAddress'.
//'nAddressLength' will hold the length of the address.
//'socket' refers to the socket who's address wil be examined.
int Socket::GetMyAddress(sockaddr_in& socketAddress, SOCKET_LEN& nAddressLength, SOCKET& socket){
  nAddressLength = sizeof(socketAddress);
  if(!getsockname(socket, (sockaddr*)&socketAddress, &nAddressLength)){
    nLastError = CSocketLib::GetLastError();  //sockets call
    return WINSOCK_ERROR;
  }
  return WINSOCK_NOERROR;
}



//This function stores the address into the reference argument 'socketAddress'.
//'nAddressLength' will hold the length of the address.
//Uses the SOCKET in my member data as the socket to get the address of.
int Socket::GetMyAddress(sockaddr_in& socketAddress, SOCKET_LEN& nAddressLength){
   return GetMyAddress(socketAddress, nAddressLength, m_Socket);
}



//The GetPeerAddress() function is much like the Winsock getpeername() function.
//  The Winsock getpeername() function is misnamed; it should be getpeeraddress().
//socketAddress will be changed to have the right addressing info, and nAddressLength
//  will be set to be the address length.
//Note that this function can be used to get the address for any socket descriptor,
//  not just our own socket descriptor.
int Socket::GetPeerAddress(sockaddr_in& socketAddress, SOCKET_LEN& nAddressLength, SOCKET& socket){
   if(socket==0) //This code only works because SOCKET is defined as a u_int and not a struct
      socket=m_Socket;
    
   nAddressLength = sizeof(socketAddress);
   if(!getpeername(socket, (sockaddr*)&socketAddress, &nAddressLength)){ //sockets call
      nLastError = CSocketLib::GetLastError();   //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}



//This version of GetPeerAddress() works on our own socket descriptor.
int Socket::GetPeerAddress(sockaddr_in& socketAddress, SOCKET_LEN& nAddressLength){
   return GetPeerAddress(socketAddress, nAddressLength, m_Socket);
}


//GetDriverWaitingSize() is much like calling ioctlsocket(s, FIONREAD,...) in Winsock.
//It returns the number of bytes waiting to be read on our socket.
//For datagrams, it is the size of the next datagram.
//For streams, it "should" be the total waiting bytes.
unsigned long Socket::GetDriverWaitingSize(){
   unsigned long lCharsWaiting;
   int           nErrorValue;

#ifdef __TV_WINDOWS__
   nErrorValue = ioctlsocket(m_Socket, FIONREAD, &lCharsWaiting);  //sockets
                                                                   //call
#else
   nErrorValue = ioctl(m_Socket, FIONREAD, &lCharsWaiting);  //sockets call
#endif
   if(nErrorValue){
      return 0L;
   }
   return lCharsWaiting;
}

unsigned long Socket::GetTotalWaitingSize(){
   return GetDriverWaitingSize();
}

int Socket::SetBroadcastOption(BOOL bBroadcast){
   if(setsockopt(m_Socket, SOL_SOCKET, SO_BROADCAST, (char*)&bBroadcast, sizeof(bBroadcast))){
      nLastError = CSocketLib::GetLastError();   //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::SetDebugOption(BOOL bDebug){
   if(setsockopt(m_Socket, SOL_SOCKET, SO_DEBUG, (char*)&bDebug, sizeof(bDebug))){
      nLastError = CSocketLib::GetLastError();   //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::SetLingerOption(BOOL bLinger, u_short nLingerTime){
//If you set 'bLinger' to TRUE, then that means to linger for 'nLingerTime' seconds.
//Examples:  bLinger=1, nLingerTime=0.      Hard immediate close. All queued data for sending gets canned immediately.
//           bLinger=1, nLingerTime=2.      Graceful close, waits 2 seconds to try to send any pending data.
//           bLinger=0, nLingerTime=<any>. "Graceful" immediate close, causes data to still be in queue to send when ready.
#ifdef __TV_WINDOWS__
  LINGER lingerOptions;
#else
  struct linger lingerOptions;
#endif

   if(bLinger)  //Note that bLinger is a BOOL and LINGER.l_onoff is a u_short.
      lingerOptions.l_onoff = 1;
   else
      lingerOptions.l_onoff = 0;
   lingerOptions.l_linger = nLingerTime;
   if(setsockopt(m_Socket, SOL_SOCKET, SO_LINGER, (char*)&lingerOptions, sizeof(lingerOptions))){  //sockets call
      nLastError = CSocketLib::GetLastError();  //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::SetRouteOption(BOOL bRoute){
//A False argument means don't route.
   bRoute = !bRoute;
   if(setsockopt(m_Socket, SOL_SOCKET, SO_DONTROUTE, (char*)&bRoute, sizeof(bRoute))){ //sockets call
      nLastError = CSocketLib::GetLastError(); //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::SetKeepAliveOption(BOOL bKeepAlive){
   if(setsockopt(m_Socket, SOL_SOCKET, SO_KEEPALIVE, (char*)&bKeepAlive, sizeof(bKeepAlive))){ //sockets call
      nLastError = CSocketLib::GetLastError();     //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::SetOOBOption(BOOL bSendOOBDataInline){
   if(setsockopt(m_Socket, SOL_SOCKET, SO_OOBINLINE, (char*)&bSendOOBDataInline, sizeof(bSendOOBDataInline))){ //sockets call
      nLastError = CSocketLib::GetLastError();  //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::SetReceiveBufferOption(int nReceiveBufferSize){
   if(setsockopt(m_Socket, SOL_SOCKET, SO_RCVBUF, (char*)&nReceiveBufferSize, sizeof(nReceiveBufferSize))){ //sockets call
      nLastError = CSocketLib::GetLastError();   //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::SetSendBufferOption(int nSendBufferSize){
   if(setsockopt(m_Socket, SOL_SOCKET, SO_SNDBUF, (char*)&nSendBufferSize, sizeof(nSendBufferSize))){ //sockets call
      nLastError = CSocketLib::GetLastError(); //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::SetReuseAddressOption(BOOL bAllowReuseAddress){
   if(setsockopt(m_Socket, SOL_SOCKET, SO_REUSEADDR, (char*)&bAllowReuseAddress, sizeof(bAllowReuseAddress))){ //sockets call
      nLastError = CSocketLib::GetLastError();  //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}

int Socket::GetBroadcastOption(BOOL& bBroadcast){
   SOCKET_LEN nSize= sizeof(bBroadcast);
   if(getsockopt(m_Socket, SOL_SOCKET, SO_BROADCAST, (char*)&bBroadcast, &nSize)){ //sockets call
      nLastError = CSocketLib::GetLastError();  //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::GetDebugOption(BOOL& bDebug){
   SOCKET_LEN nSize= sizeof(bDebug);
   if(getsockopt(m_Socket, SOL_SOCKET, SO_DEBUG, (char*)&bDebug, &nSize)){ //sockets call
      nLastError = CSocketLib::GetLastError();   //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::GetLingerOption(BOOL& bLinger, u_short& nLingerTime){
#ifdef __TV_WINDOWS__
  LINGER lingerOptions;
#else
  struct linger lingerOptions;
#endif
   SOCKET_LEN nSize= sizeof(lingerOptions);

   if(getsockopt(m_Socket, SOL_SOCKET, SO_LINGER, (char*)&lingerOptions, &nSize)){ //sockets call
      nLastError = CSocketLib::GetLastError(); //sockets call
      return WINSOCK_ERROR;
   }
   bLinger = lingerOptions.l_onoff;
   nLingerTime = lingerOptions.l_linger;
   return WINSOCK_NOERROR;
}
int Socket::GetRouteOption(BOOL& bRoute){
   SOCKET_LEN nSize= sizeof(bRoute);
   if(getsockopt(m_Socket, SOL_SOCKET, SO_DONTROUTE, (char*)&bRoute, &nSize)){  //sockets call
      nLastError = CSocketLib::GetLastError();  //sockets call
      return WINSOCK_ERROR;
   }
   //bRoute value of TRUE means "don't route."  So we change it to mean what you'd think.
   bRoute = !bRoute;
   return WINSOCK_NOERROR;
}
int Socket::GetKeepAliveOption(BOOL& bKeepAlive){
   SOCKET_LEN nSize= sizeof(bKeepAlive);
   if(getsockopt(m_Socket, SOL_SOCKET, SO_KEEPALIVE, (char*)&bKeepAlive, &nSize)){ //sockets call
      nLastError = CSocketLib::GetLastError(); //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::GetOOBOption(BOOL& bSendOOBDataInline){
   SOCKET_LEN nSize= sizeof(bSendOOBDataInline);
   if(getsockopt(m_Socket, SOL_SOCKET, SO_OOBINLINE, (char*)&bSendOOBDataInline, &nSize)){ //sockets call
      nLastError = CSocketLib::GetLastError();  //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::GetReceiveBufferOption(int& nReceiveBufferSize){
   SOCKET_LEN nSize= sizeof(nReceiveBufferSize);
   if(getsockopt(m_Socket, SOL_SOCKET, SO_RCVBUF, (char*)&nReceiveBufferSize, &nSize)){ //sockets call
      nLastError = CSocketLib::GetLastError();  //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::GetSendBufferOption(int& nSendBufferSize){
   SOCKET_LEN nSize= sizeof(nSendBufferSize);
   if(getsockopt(m_Socket, SOL_SOCKET, SO_SNDBUF, (char*)&nSendBufferSize, &nSize)){ //sockets call
      nLastError = CSocketLib::GetLastError();   //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}
int Socket::GetReuseAddressOption(BOOL& bAllowReuseAddress){
   SOCKET_LEN nSize= sizeof(bAllowReuseAddress);
   if(getsockopt(m_Socket, SOL_SOCKET, SO_REUSEADDR, (char*)&bAllowReuseAddress, &nSize)){ //sockets call
      nLastError = CSocketLib::GetLastError();   //sockets call
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}



// I/O
int Socket::SetBlockingMode(int blockMode)
{
  //
  // set the socket to blocking mode.
#ifdef __TV_UNIX__
  int iost = ioctl(m_Socket, FIONBIO, &blockMode);
  return iost;
#endif
#ifdef __TV_WINDOWS__
	u_long bm = blockMode;
  int iost = ioctlsocket(m_Socket, FIONBIO, &bm);
  return iost;
#endif
}

