/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <assert.h>
#include <stdio.h>
#include <fstream.h>

//#include <hashdict.h>
#include <collstr.h>
#include <RVCTokenizer.h>
#include <set.h>
#include <regexp.h>

//#include "associativarray.h"
#include "tvdebug.h"
#include "tvtables.h"
#ifndef __TV_UNIX__
#include "tvver.h"
#endif
#include "tvenv.h" 
//#include "tracedll.h"

#ifdef __TV_UNIX__
#include <sys/utsname.h>
#endif

/*PTR_DOC::
#ifdef __TV_WINDOWS__
#include "psapi.h"
#endif
*/
static TableHandle IniTable;
static RVCString TVEnvErrorString = "";

// NOTE: size of table is limited to 1024 pairs, each key/value is limited to 256 bytes
typedef struct
{
	char key[256];
	char value[256];
} S_KEY_VALUE_PAIR;

#pragma data_seg( ".SharedData" )
static S_KEY_VALUE_PAIR g_volatileEnvTable[1024] = { 0 };
static unsigned         g_uTableSize = 0;
#pragma data_seg()

//---------------------------------------------------------------------------
// addReplaceEnvValue - replace existing value or add new key-value pair
//---------------------------------------------------------------------------
static void addReplaceEnvValue( const char* key, const char* value )
{
	// if key already exists - replace it's value
	for ( unsigned c = 0; c < g_uTableSize; c++ )
	{
		if ( strcmp( key, g_volatileEnvTable[c].key ) == 0 )
		{
			strcpy( g_volatileEnvTable[c].value, value );
			return;
		}
	}

	// add new key and value
	strcpy( g_volatileEnvTable[ g_uTableSize ].key, key );
	strcpy( g_volatileEnvTable[ g_uTableSize ].value, value );

	//WlTraceMsg( "added: %s, %s; length: %d", g_volatileEnvTable[ g_uTableSize ].key, g_volatileEnvTable[ g_uTableSize ].value, g_uTableSize+1 );
	g_uTableSize++;
}

//---------------------------------------------------------------------------
// getEnvValue - retrieve a key's value - if exist
//---------------------------------------------------------------------------
static const char* getEnvValue( const char* key )
{
	for ( unsigned c = 0; c < g_uTableSize; c++ )
	{
//		if ( strcmp ( key, "JRP_PLAY_MODE" ) == 0 )
//			WlTraceMsg( "comparing %s to %s", key, g_volatileEnvTable[c].key );

		if ( strcmp( key, g_volatileEnvTable[c].key ) == 0 )
		{
			return g_volatileEnvTable[c].value;
		}
	}

	return NULL;
}






void WINAPI TVPFatalErrorLog(const char* format,...)
{
#ifdef _DEBUG
  char errbuf[2048];
  va_list ap;
  va_start(ap, format);
  vsprintf(errbuf, format, ap);
  va_end(ap);

  TVEnvContainerClass tvpc;
  RVCString logfile = tvpc.TVPResolveFilePathNoTVEnv("webload.log", "HOME");
  ofstream f(logfile, ios::app);
  f << errbuf << endl;
  f.close();
#endif
}



//******************************************************************************
//                            Class   AddDel
//******************************************************************************
// The class adds a variable to a global table during construction, removes it
// during destruction. failed() returns TRUE if the value is already in the
// table. The class is used to detect infinite loop during recursive calls to
// the resolve functions.
class AddDel
{
 private :

    RVCollectableString* string;
    static RVSet set;
    BOOL fail;

 public :

    AddDel(const char* str);
    ~AddDel();
    BOOL failed() { return fail;}
};

//------------------------------------------------------------------------------
AddDel::AddDel(const char* str)
{
 fail = FALSE;
 string = new RVCollectableString(str);
 if (string!=set.insert(string))
    fail=TRUE;
}
//------------------------------------------------------------------------------
AddDel::~AddDel()
{
 set.remove(string);
 delete string;
}
//------------------------------------------------------------------------------

RVSet AddDel::set; // the global table used by the class.

//******************************************************************************
//                  TVS  Resolve functions
//******************************************************************************
// replace #{value} and ${value} with the appropriate environment values.
static BOOL TVSLoadResolve(RVCString& str)
{
	size_t i,j;
	const char* env;
	AddDel adddel(str.data());

	if (adddel.failed())
	  return FALSE;

// nikitin 2007/01/31
//	while ((i = str.index(RVCRegexp("[$#]{[^ \t\n()]+}"),&j))!=RV_NPOS) {
//	while ((i = str.index(RVCRegexp("[$#]{[^ \t\n)]+}"),&j))!=RV_NPOS) {
	while ((i = str.index(RVCRegexp("[$#][{]([^\\t\n()]+)[}]"),&j))!=RV_NPOS) {
		if (str[i]=='$')
			env = getenv(RVCString(str(i+2,j-3)));
		else
			env = TVSGetValue(IniTable, RVCString(str(i+2,j-3)));

    env = env ? env : "";
    RVCString temp(env);

		if (TVSLoadResolve(temp))
			str.replace(i,j,temp);
		else
			return FALSE;
	}
	return TRUE;
}

//------------------------------------------------------------------------------
// replace #(value) and $(value) with the appropriate environment values.
BOOL TVSRuntimeResolve(RVCString& str)
{
	size_t i,j;
	const char* env;
	AddDel adddel(str.data());

	if (adddel.failed())
		 return FALSE;

// nikitin 2007/01/31
//	while ((i = str.index(RVCRegexp("[$#]([^ \t\n()]+)"),&j))!=RV_NPOS){
//	while ((i = str.index(RVCRegexp("[$#]([^ \t\n )]+)"),&j))!=RV_NPOS){
	while ((i = str.index(RVCRegexp("[$#][(]([^\\t\\n()]+)[)]"),&j))!=RV_NPOS){
		if (str[i]=='$')
			env = getenv(RVCString(str(i+2,j-3)));
		else
  		env = TVSGetValue(IniTable, RVCString(str(i+2,j-3)));

    env = env?env:"";
    RVCString temp(env);
		if (TVSRuntimeResolve(temp))
			str.replace(i,j,temp);
		else
			return FALSE;
	}
	return TRUE;
}

static const char* TVSLoadFilter(const char* key,const char* value)
{
  static RVCString resolvedVal;
  
  resolvedVal = value ? value : "";

  if (!TVSLoadResolve(resolvedVal)) {
	   TVEnvErrorString.append("\n endless loop while resolving value ");
	   return NULL;
  }
  return (const char*)resolvedVal;
}

BOOL WINAPI TVEInit(const char *tvpath)
{
  EnterProcedure("TVEInit");
  assert(tvpath);
  TVEnvContainerClass tvpc;
#ifndef __TV_UNIX__
	RVCString IniFileName = tvpc.TVPFindfileNoTVEnv(TVVProductIniFile(), tvpath, "$TV_BIN", "HOME", NULL);
	if (IniFileName.isNull()) {
  	TVPFatalErrorLog("tvenv.dll: %s not found", TVVProductIniFile());
    return FALSE;
  }
#else

  const char *IniFile;
  if( (IniFile = tvpc.TVPFindfileNoTVEnv("webload.ini", tvpath, "$TV_BIN",
                                          "HOME", NULL)) == NULL  ) {
    TVPFatalErrorLog("WLEnvironment: webload.ini not found");
    return FALSE;
  }
  RVCString IniFileName = IniFile;
#endif

	////sergei 3/24/01 - debugPrint(3,("Loading main file : '%s'\n",IniFileName));
	IniTable = TVSCreateTable();
	if (!TVSLoad(IniFileName,IniTable,TVSLoadFilter)) {
    char msg[32 + MAX_PATH];
    sprintf(msg, "\n Failed to load %s\n The file contains a syntax error", IniFileName.data());
		TVEnvErrorString.append(msg);
		TVSDestroyTable(IniTable);
		return FALSE;
	}
  return TRUE;
}

DllExport const char* WINAPI TVEError()
{
  return (const char*)TVEnvErrorString;
}

const char* WINAPI TVEnvContainerClass::TVEGetEnv( const char *entry )
{
  return TVERuntimeResolve( TVSGetValue( IniTable, entry ) );
}

const char* WINAPI TVEnvContainerClass::TVEGetEnvValue( const char *entry )
{
	// first look in the local table
#ifdef _DEBUG
	FILE* f = fopen( "c:\\temp\\env.txt", "ab" );
	char line[1024] = "";
	if ( f )
	{
		sprintf( line, "%08x - %d, get env name: %s", GetCurrentThreadId(), g_uTableSize, entry );
		fwrite( line, strlen(line), 1, f );
		fclose( f );
	}
#endif
	//WlTraceMsg( "TVEnvContainerClass::TVEGetEnvValue - key: %s", entry );
	const char* value = getEnvValue( entry );
#ifdef _DEBUG
	f = fopen( "c:\\temp\\env.txt", "ab" );
	if ( f )
	{
		if ( value )
			sprintf( line, ", value: %s\n", value );
		else
			sprintf( line, ", value: (null)\n" );
		fwrite( line, strlen(line), 1, f );
		fclose( f );
	}
#endif

	if ( value )
	{
//		WlTraceMsg( "TVEnvContainerClass::TVEGetEnvValue - value: %s", value );
		return value;
	}

  return TVEGetEnv( entry );
}

// add new entry and value to the table
// if entry already exist, replace it's value
void WINAPI TVEnvContainerClass::TVESetEnvValue( const char* entry, const char* value )
{
#ifdef _DEBUG
	FILE* f = fopen( "c:\\temp\\env.txt", "ab" );
	char line[1024] = "";
	if ( f )
	{
		sprintf( line, "%08x - set env name: %s, value: %s\n", GetCurrentThreadId(), entry, value );
		fwrite( line, strlen(line), 1, f );
		fclose( f );
	}
#endif
	
//	WlTraceMsg( "TVEnvContainerClass::TVESetEnv - key: %s, value: %s", entry, value );
	addReplaceEnvValue( entry, value );
}

bool WINAPI TVEnvContainerClass::TVEEnvDefined(const char *entry,
																							 const char* value,
																							 bool ignore_case /* =true*/)
{
  const char *pEnvOption = TVEGetEnv(entry);
  if (pEnvOption!=NULL) {
    RVCString EnvOption = pEnvOption ;
    RVCString::caseCompare cmp = RVCString::ignoreCase;
    if (!ignore_case)
			cmp = RVCString::exact; 
    if (EnvOption.index(value ,0, cmp ) != RV_NPOS)
			return true;
  }
	
	return false;
}

const char* WINAPI TVEnvContainerClass::TVERuntimeResolve(const char *str)
{
  if (!str)
    return NULL;

  RVCString value = str;
  BOOL ret = TVSRuntimeResolve(value);

  if (ret) {
    strcpy(m_TVERuntimeResolve_retVal, (const char*)value);
    return m_TVERuntimeResolve_retVal;
  }
  return NULL;
}


const char* WINAPI TVPGetOSInfo(){
  const char *platform;
  static char result[256];

#ifdef __TV_WINDOWS__
  // Get Windows version
  OSVERSIONINFO verInfo;
  verInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
  GetVersionEx(&verInfo);
  
  switch (verInfo.dwPlatformId){
  case VER_PLATFORM_WIN32s:
    platform = "Windows 3.1";
    break;
  case VER_PLATFORM_WIN32_WINDOWS:
    if (verInfo.dwMinorVersion)
      platform = "Windows 98";
    else
      platform = "Windows 95";
    break;
  case VER_PLATFORM_WIN32_NT:
    platform = "Windows NT/Windows 2000";
    break;
  default:
    platform = "Windows";
  }

  sprintf(result,"%s (%s)", platform, verInfo.szCSDVersion);

#else
#ifdef __TV_solaris__
  struct utsname name;
  uname(&name);
  sprintf(result, "Solaris (%s %s)", name.sysname, name.release);

#else
#ifdef __TV_linux__
  struct utsname name;
  uname(&name);
  sprintf(result, "Linux (%s %s)", name.sysname, name.release);

#else
  strcpy(result, "Unknown");

#endif // __TV_linux__

#endif // __TV_solaris__

#endif // __TV_WINDOWS__

  return result;
  
}

// @@Chaim - not declared in h

bool IsNT()
{	
//	#if 1
#ifdef __TV_WINDOWS__
	OSVERSIONINFO vi = { sizeof(vi)};	
	::GetVersionEx(&vi);
	return vi.dwPlatformId == VER_PLATFORM_WIN32_NT;
	#else
	return false;
	#endif
}

HANDLE GetProcessHandle()
{
#ifdef __TV_WINDOWS__
	if (IsNT())
		return GetCurrentProcess();
	else
		return (HANDLE)GetCurrentProcessId();
#else
          return NULL;
#endif
}

size_t WINAPI TVPGetWorkingSetSize()
{
  return -1;
}
